/******************************************************************************\
*  Copyright (C) 1995 Dallas Semiconductor Corporation.                        *
*  All rights Reserved. Printed in U.S.A.                                      *
*  This software is protected by copyright laws of                             *
*  the United States and of foreign countries.                                 *
*  This material may also be protected by patent laws of the United States     *
*  and of foreign countries.                                                   *
*  This software is furnished under a license agreement and/or a               *
*  nondisclosure agreement and may only be used or copied in accordance        *
*  with the terms of those agreements.                                         *
*  The mere transfer of this software does not imply any licenses              *
*  of trade secrets, proprietary technology, copyrights, patents,              *
*  trademarks, maskwork rights, or any other form of intellectual              *
*  property whatsoever. Dallas Semiconductor retains all ownership rights.     *
*                                                                              *
*  File: child1.cpp                                                            *
*  Include file for exp1.cpp                                                   *
\******************************************************************************/

// ===========================================================
// The MDI child window class
// ===========================================================
class TDecoChild: public TMDIChild {
public:

   time_t timer;                                   //variable to get system time
   struct tm *tblock;
      
   TDecoChild(TMDIClient& parent, const char far *name);           //constructor
   ~TDecoChild();
      
   void  SetupWindow();
   void  ChangeName(const char far *name);
   int   Unnamed() { return filename.length() == 0; }

   int   SaveFile();                 //function to save log file
   void  EvTimer(UINT timerId);      //timer function to poll microlan
   int   handle;                     //handle for TMEX
   int   numRows;                    //variable to store the number of log entries
   string filename;                  //logfile name

private:
 
   TListBox* listbox;                //pointer to the listbox
   TStatic*  title1;                 //pointer to static text
   TStatic*  title2;                 //pointer to static text
   TStatic*  time_text;              //pointer to static text to hold time info
   TIArrayAsVector<string> sArray;   //array to hold log text
   int last_pass; // flag to indicate if network has changed since last pass;
   int last_szmissing;
   DECLARE_RESPONSE_TABLE(TDecoChild);
};
//==============================================================================
// Response Table
//==============================================================================
DEFINE_RESPONSE_TABLE1(TDecoChild, TMDIChild)
   EV_WM_TIMER,
END_RESPONSE_TABLE;

//==============================================================================
// Constructor
//==============================================================================
TDecoChild::TDecoChild(TMDIClient& parent, const char far *name)
  : TMDIChild(parent, name), sArray(SARRAY_HIGH, SARRAY_LOW, SARRAY_DELTA)
{
   filename = name;                           //get the logfile name
   Attr.Style |= MDIS_ALLCHILDSTYLES;         //allow any child window styles
   Attr.Style |=WS_MAXIMIZE;                  //maximize the window

   //set up the static text boxes
   title1 = new TStatic(this, -1, "Name",50, 25, 50, 20);
   title2 = new TStatic(this, -1, "Status at", 250, 25, 60, 20);
   time_text = new TStatic(this, -1, "", 325, 25 , 175, 20);

   numRows = 0;
   num_waits = 0;
   last_szmissing = 0;
   last_pass = TRUE;

   TModule *TM;       //needed for timer
   TM = GetModule();
   // timer for saving file every few minutes.  
   if(strcmp(name, "") != 0)
      ID_MYTIMER = ::SetTimer(NULL, 1, 31000, (TIMERPROC)MakeProcInstance((FARPROC)Save, TM->GetInstance())); 
}

//==============================================================================
// Destructor
//==============================================================================
TDecoChild::~TDecoChild()
{
   if(title1)
      delete title1;
   if(title2)
      delete title2;
   if(time_text)
      delete time_text;
   if(listbox)
      delete listbox;
}

//==============================================================================
// Save: Windows timer function 
//==============================================================================
WORD FAR PASCAL Save(HWND, WORD, WORD, LONG)
{
   cp->SaveFile();
   return 0;
}

//==============================================================================
// SaveFile:  Called on exit and by the Save function
//==============================================================================
int TDecoChild::SaveFile()
{
   char c[80];
   int len;
   const char* cst;
   if(timecount == 0)  //check to see if the save interval is up;
   {
      if(numRows)   //check to make sure file needs to be saved.  
      {
         FILE *fp;      //define file pointer
         fp = fopen(filename.c_str(), "a+");  //attempt to open the log file
         if(fp != NULL)
         {
            MessageBeep(-1);
            for(int i = 0; i < numRows; i++)   
            {   
               cst = sArray[i]->c_str();
               strcpy(c, cst);
               c[(len = strlen(c))] = 0x0a;  //add carriage return
               c[len+1] = 0;   //terminate with a zero
               fwrite(c, strlen(c), 1, fp);    //write to the file
            }
            timecount = WAIT_TIME; //reset save delay
            fclose(fp);           //close file pointer
            sArray.Flush();     //flush the file data buffer
            numRows = 0;        //reset the file data length buffer
         }
         else  //error on file open, reset delay
         {
            timecount = WAIT_TIME;
            MessageBox("Unable to open log file", "ERROR", MB_ICONSTOP + MB_OK);
         }
      }
      else  //nothing to save
         timecount = WAIT_TIME;  //reset the save delay
   }
   else                        //if not zero, decrement timecount;
      --timecount;
   return TRUE;
}

//==============================================================================
// EvTimer: class timer to poll the system
//==============================================================================
void TDecoChild::EvTimer(UINT)   
{
   char s[640];
   int i;
   int j;
    
   if(kill_timer_flag) //check to see if need to kill the timers
   {
      KillTimer(NULL);
      ::KillTimer(NULL, ID_MYTIMER);
      return;
   }
   
   if(num_waits == 0)  //make sure right amount of time has elapsed
   {
     //   szmissing[0] = 0;           
      if(Check_Tree())           //check for missing devices
      {
         //determine which logging procedure is in use.
         if((log_pass_flag == 2 && last_pass == TRUE) || log_pass_flag == 0)
         {
            timer = time(NULL);           //get time 
            tblock = localtime(&timer);
            strcpy(s, asctime(tblock));
            s[strlen(s) - 1] = 0;         //clean up time string
            time_text->SetText(s);        //set the text to time
            sArray.Add(new string(s));
            numRows++;
                     
            
            
            last_pass = FALSE;

            for( i = 0; i < 40; i++)
               strcpy(device_array[i], ""); //clear the array

            fill_array_with_info(root);    //get status of each device
            listbox->ClearList();          //clear the listbox
            for(i = 0; i < 40; i++)        //insert the strings into the listbox
               listbox->InsertString(device_array[i], i);



for(i = 0; i < num_devices; i++) //copy the status of each device
            {                                //to the array to be saved
               strcpy(s, device_array[i]);
               for(j = strlen(s); j < 25; j++)
                  s[j] = ' ';
               s[25] = 0;
               strcat(s, device_array[i+20]);
               sArray.Add(new string(s));
               numRows++;          //increment the line number for save function
            }
            strcpy(s, " ");           //Add a blank line to the log file
            sArray.Add(new string(s));
            numRows++;


            last_szmissing = szmissing = 0;
          }
      }
      else
      {
            //determing which logging procedure is in use;
         if((log_pass_flag == 2 && (last_pass == FALSE || last_szmissing != szmissing)) || log_pass_flag == 1 || log_pass_flag == 0)
         {
            timer = time(NULL);
            tblock = localtime(&timer);
            strcpy(s, asctime(tblock));
            s[strlen(s) - 1] = 0;
            time_text->SetText(s);
            numRows++;
            sArray.Add(new string(s));
                     
            fill_array_with_info(root);
            listbox->ClearList();
            for(i = 0; i < 40; i++)
               listbox->InsertString(device_array[i], i);
            last_szmissing = szmissing;
            
            for(i = 0; i < num_devices; i++)
            {
               numRows++;
               strcpy(s, device_array[i]);
               for(int j = strlen(s); j < 25; j++)
                  s[j] = ' ';
               s[25] = 0;
               strcat(s, device_array[i+20]);
               sArray.Add(new string(s));
            }
            strcpy(s, " ");
            numRows++;
            sArray.Add(new string(s));
            last_pass = TRUE;
            for( i = 0; i < 40; i++)
               strcpy(device_array[i], "");

            szmissing = 0;
         }
         szmissing = 0;
      }
      num_waits = old_num_waits;
   }
   else
      num_waits--;
}

//==============================================================================
// SetupWindow:
//==============================================================================
void TDecoChild::SetupWindow()
{
   TEXTMETRIC tm;
   int tab_pos[2];

   tab_pos[0] = 0;                    //setup listbox columns
   tab_pos[1] = 200;
   dc = new TClientDC(*this);         //get device context
   dc->GetTextMetrics(tm);            //get text info
   int cyChar = tm.tmHeight + tm.tmExternalLeading; //determine height of text

   delete dc;                     //delete device context as quickly as possible
   
   SetTimer(NULL, global_time_interval*1000);   //initialize log pass timer
   timer_set_flag = TRUE;

   listbox = new TListBox(this, ID_LISTBOX, 50, 50, 420, cyChar*20);   

   listbox->Attr.Style |= LBS_MULTICOLUMN;   //multicolumn listbox
   listbox->Attr.Style &= ~WS_HSCROLL;       //no horizontal scrollbar
   TFrameWindow::SetupWindow();   //must call before setting listbox widths and
                                  //tab stops
   listbox->SetColumnWidth(200);
   listbox->SetTabStops(2, tab_pos);
}


