/******************************************************************************\
*  Copyright (C) 1995 Dallas Semiconductor Corporation.                        *
*  All rights Reserved. Printed in U.S.A.                                      *
*  This software is protected by copyright laws of                             *
*  the United States and of foreign countries.                                 *
*  This material may also be protected by patent laws of the United States     *
*  and of foreign countries.                                                   *
*  This software is furnished under a license agreement and/or a               *
*  nondisclosure agreement and may only be used or copied in accordance        *
*  with the terms of those agreements.                                         *
*  The mere transfer of this software does not imply any licenses              *
*  of trade secrets, proprietary technology, copyrights, patents,              *
*  trademarks, maskwork rights, or any other form of intellectual              *
*  property whatsoever. Dallas Semiconductor retains all ownership rights.     *
*                                                                              *
*  Compiler: Borland C++ 4.5  OWL                                              *
*  Ver 1.0                                                                     *
*  Date: 4/27/95                                                               *
*                                                                              *
\******************************************************************************/

// =============================================================================
// Include files
// =============================================================================
#include <fcntl.h>
#include <sys\stat.h>
#include <math.h>
#include <owl\static.h>
#include <string.h>
#include <stdlib.h>
#include <owl\applicat.h>
#include <owl\framewin.h>
#include <owl\dc.h>
#include <owl\dialog.h>
#include <owl\opensave.h>
#include <owl\listbox.h>
#include <windows.h>
#include <owl\scroller.h>
#include <owl\point.h>
#include <owl\dc.h>
#include <classlib\arrays.h>
#include <classlib\stdtempl.h>
#include <cstring.h>
#include <owl\color.h>
#include <owl\gdiobjec.h>
#include <stdio.h>
#include <owl\decframe.h>
#include <owl\controlb.h>
#include <owl\gadget.h>
#include <owl\buttonga.h>
#include <owl\messageb.h>
#include <owl\toolbox.h>
#include <owl\statusba.h>
#include <time.h>
#include <bwcc.h>
#include <owl\decmdifr.h>
#include <owl\mdi.h>
#include <owl\mdichild.h>
#include <owl\mdi.rh>
#include <mem.h>
#include <owl\owlpch.h>
#pragma hdrstop

// =============================================================================
// defines
// =============================================================================
#define MAXTIMEINTERVAL 60.0
#define MINTIMEINTERVAL 0.0
#define MAX_LINES 48
#define SARRAY_HIGH 249
#define SARRAY_LOW 0
#define SARRAY_DELTA 125
#define TICK_SEG 0x40
#define TICK_OFF 0x6c
#define TRUE 1
#define FALSE 0

#define LINE1 "Window\Door Monitor Test Program"
#define LINE2 ""
#define LINE3 "Dallas Semiconductor"
#define LINE4 "Copyright 1995"
#define WAIT_TIME 5

// =============================================================================
// my include files;
// =============================================================================
#include "define1.h"
#include "exp1.h"
#include "tree1.cpp"       //binary tree and miscellaneos functions  
#include "dialog1.cpp"     //general dialog classes
#include "child1.cpp"      //child window class



// =============================================================================
// The MDI client window class
// =============================================================================

class TMicroLanClient: public TMDIClient {
public:

   TMicroLanClient(): TMDIClient() {};  
  
  TFileOpenDialog::TData *data;   //common file open dialog
  
protected:
   void CmFileOpen();
   void CmHelpAbout();                                       
   void CmLogOn();
   void CmLogOff();
   void CmSystem();
   void CmDiagnostics();
   void CmExit();
   void CmComSetup();
   void SetupWindow();
   void Add_Device();
DECLARE_RESPONSE_TABLE(TMicroLanClient);
};
// =============================================================================
// Response Table
// =============================================================================

DEFINE_RESPONSE_TABLE1(TMicroLanClient, TMDIClient)
  EV_COMMAND(CM_FILEOPEN, CmFileOpen),
  EV_COMMAND(CM_HELPABOUT, CmHelpAbout),
  EV_COMMAND(CM_LOGSETUP, CmSystem),
  EV_COMMAND(CM_LOGON, CmLogOn),
  EV_COMMAND(CM_LOGOFF, CmLogOff),
  EV_COMMAND(CMEXIT, CmExit),
  EV_COMMAND(CM_COMSETUP, CmComSetup),
  EV_COMMAND(CM_DIAGNOSTICS, CmDiagnostics),
END_RESPONSE_TABLE;

// =============================================================================
// Setup Window
// =============================================================================
void TMicroLanClient::SetupWindow()
{
   TWindow::SetupWindow();
   
   uchar k;
   if(comport == 0)
   {
      for(int i = 1; i < 5; i++)
      {
         handle = TMStartSession(i);
         k = TMSetup(handle);
         if(k == 1)
         {
            comport = i;
            break;
         }
      }
   }
   else
   {
      handle = TMStartSession(comport);
      k = TMSetup(handle);
   }
   TDialog* TPrev = new TPreviousDlg(this, PREVIOUS); 
     TPrev->Execute();

   if(previous_flag)
   {                  //read in previous session settings
      in_stream = open("exp1.stp", O_RDWR | O_BINARY, S_IREAD | S_IWRITE);
      if(in_stream < 1)
         Add_Device();
      else
      {
         read_settings();
         agetree(root);
         close(in_stream);
         fill_array_with_info(root);
      }
   }
   else
      Add_Device();
   
}

// =============================================================================
// Add_Device: Add devices from the microlan
// =============================================================================
void TMicroLanClient::Add_Device()
{
   int found_one;              //found a new device;
   root = NULL;                //root of binary tree = 0;
   rom[0] = 0x00;              //set lsb of rom to 0 to get romid
   struct tnode far * struct1;
   TDialog* TRen = new TRenameDlg(this, REN_ID);
   TDialog* TAdd = new TAddDlg(this, ADD_ANOTHER);

   age_flag = 1;
   num_devices = 0;                       //no devices found yet

   TAdd->Execute();                    //Call add device dialog
   TMEndSession(handle);
   handle = TMStartSession(comport);
   TMSetup(handle);
   if(TMFirst(handle, GB))             //search port for devices
   {
      TMRom(handle, GB, rom);          //get romid
      if(rom[0] == DS2401)               //check for family code of DS2401
      {
         TRen->Execute();              //call rename device dialog
         if(Ren_ok_flag)               //make sure cancel wasn't hit
         {
            root = addtree(root, rom);    //add the device to the btree
            num_devices++;                //increment the device variable
            TAdd->Execute();              //add another device?
         }
         else PostQuitMessage(0);
      }
   }

   while(add_device)                     //add devices until 20 or cancel is hit
   {
      while(TRUE)
      {
         if(!TMValidSession(handle))
         {
            TMEndSession(handle);
            handle = TMStartSession(comport);
            TMSetup(handle);
         }
         found_one = FALSE;
         
         if(TMFirst(handle, GB))
         {
            rom[0] = 0;
            TMRom(handle, GB, rom);
            if(rom[0] == DS2401)
            {
               struct1 = search_tree(root, rom);
               if(struct1 == NULL)
               {
                  found_one = TRUE;
                  TRen->Execute();
                  if(Ren_ok_flag)
                  {
                     addtree(root, rom);
                     num_devices++;
                     TAdd->Execute();
                  }
                  if(!add_device || num_devices > 20)
                     break;
               }   
            }
         }
         while(TMNext(handle, GB))
         {
            rom[0] = 0;
            TMRom(handle, GB, rom);
            if(rom[0] == DS2401)
            {
               struct1 = search_tree(root, rom);
               if(struct1 == NULL)
               {
                  found_one = TRUE;
                  TRen->Execute();
                  if(Ren_ok_flag)
                  {
                     addtree(root, rom);
                     num_devices++;
                     TAdd->Execute();
                     break;
               
                  }
                  if(!add_device || num_devices > 20)
                     break;
               }
            }
            if(!TMValidSession(handle))
            {
               TMEndSession(handle);
               handle = TMStartSession(comport);
               TMSetup(handle);
            }
         }
         if(!found_one)
         {
            add_device = FALSE;
            break;
         }
         else
            found_one = FALSE;
      }
      if(!add_device || num_devices > 20)
         break;   
   }
   agetree(root);
   return;
}

// =============================================================================
// CmFileOpen: Open the child window class
// =============================================================================
void TMicroLanClient::CmFileOpen()
{
  cp = new TDecoChild(*this, filen);  //execute the child window class
  cp->Create();
}

// =============================================================================
// CmSystem:  System Settings dialog
// =============================================================================
void TMicroLanClient::CmSystem()
{
   TDlgDialog TD(this, DIALOG_2);
   int i;
   i = TD.Execute();
   if((log_file_open_flag == 1) && (i != IDCANCEL))        //check to see if saving to log file
   {
      data = new TFileOpenDialog::TData(
         OFN_HIDEREADONLY, "LOG files (*.log)|*.log|",
         0,
         0,
         "LOG"
      );
      if ((new TFileOpenDialog(this, *data))->Execute() == IDOK)
      {
         strcpy(filen, data->FileName);
         if(cp != NULL)
         {
            cp->SetCaption(filen);
            TModule *TM;       //needed for timer
            TM = GetModule();
            cp->filename = filen;
            ID_MYTIMER = ::SetTimer(NULL, 1, 31000, (TIMERPROC)MakeProcInstance((FARPROC)Save, TM->GetInstance()));
         }
      }
      
   }
   else if(log_file_open_flag == 0 && i != IDCANCEL)
   {
      strcpy(filen, ""); // no log file save
      if(cp && ID_MYTIMER != NULL)
      {
         timecount = 0;
         cp->SetCaption("");
         cp->SaveFile();           //save the log file if it needs it
         ::KillTimer(NULL, ID_MYTIMER); //kill the save timer
         ID_MYTIMER = 0;
      }
   }
}

// =============================================================================
// CmDiagnostics:  Perform system diagnostics
// =============================================================================
void TMicroLanClient::CmDiagnostics()
{
   int i;
   if(!TMValidSession(handle))
   {
      TMEndSession(handle);
      handle = TMStartSession(comport);
      i = TMSetup(handle);
   }
   else
      i = TMSetup(handle);
   if(i == 1)
      BWCCMessageBox(NULL, "System OK", "Diagnostics", MB_ICONINFORMATION);
   else if(i == 2)
      BWCCMessageBox(NULL, "The MicroLan is shorted or the Com Port adapter is not connected.", "Diagnostics", MB_ICONHAND);
}

// =============================================================================
// Display help message box
// =============================================================================
void TMicroLanClient::CmHelpAbout()
{
   (new TAboutDialog(this, DIALOG_1))->Execute();
}

// =============================================================================
// CmLogOn:  begin logging of microlan, call CmFileOpen
// =============================================================================
void TMicroLanClient::CmLogOn()
{
   if(!already_loging) //only one log session at a time
   {
      CmFileOpen();
      already_loging = TRUE;
      kill_timer_flag = FALSE;
   }
}

// =============================================================================
// CmLogOff:  Turn off log file and destroy child window
// =============================================================================
void TMicroLanClient::CmLogOff()
{
   if(cp)          //make sure child exists, gp fault otherwise
   {
      if(ID_MYTIMER != NULL)       //check to see if saving to file was on
      {
         log_file_open_flag = 0;
         strcpy(filen, "");
         timecount = 0;
         cp->SaveFile();           //save the log file if it needs it
         ::KillTimer(NULL, ID_MYTIMER); //kill the save timer
         ID_MYTIMER = FALSE;
      }
      cp->SetCaption("");
   }
}

// =============================================================================
// CmExit:  Exit the program
// =============================================================================
void TMicroLanClient::CmExit()
{
   out_stream = open("exp1.stp", O_CREAT | O_TRUNC | O_RDWR | O_BINARY, S_IREAD|S_IWRITE);
   if(out_stream > 0)
      save_setup(root);
   close(out_stream);
   if(timer_set_flag)           //make sure timers were killed
      cp->KillTimer(NULL);
   if(ID_MYTIMER != NULL)
   {
      ::KillTimer(NULL, ID_MYTIMER);
      timecount = 0;
      cp->SaveFile();
   }
   deltree(root);
   PostQuitMessage(0);
}
// =============================================================================
// CmComSetup:  Allow to change comports; not very useful unless 
// add_device called again
// =============================================================================
void TMicroLanClient::CmComSetup()
{
   char testbuff[10];
  
   TComSetupDlg TCom(this, DIALOG_4);
   if(TCom.Execute() == IDOK)
   {
      itoa(comport, testbuff, 10);
      testbuff[1] = '0';
      BWCCMessageBox(NULL, testbuff, "ComPort", MB_OK);
   }
}

// ===========================================================
// The MDI application class
// ===========================================================

class TMicroLanApp: public TApplication {
public:
  TMicroLanApp(const char far* name)
    : TApplication(name) {};
  void InitMainWindow();
  bool CanClose();
private:
  TMicroLanClient* client;
  TControlBar* toolbar;
  TStatusBar* statusLine;
};

// =============================================================================
// InitMainWindow: setup main window
// =============================================================================
void TMicroLanApp::InitMainWindow()
{
  
// Construct the MDI client window
  client = new TMicroLanClient;

// Construct MDI frame, passing client window object
////////////////////////////////////////////////////////////////

  TDecoratedMDIFrame * frame = new TDecoratedMDIFrame(
    "Window/Door Monitor with DS2401",  // Caption
    MENU_1,      // Required menu resource identifier
    *client,      // Required client window object
    TRUE);        // Optional track menu selections


// Construct a toolbar with icons for various commands
  toolbar = new TControlBar(frame);
  toolbar->Insert(
   *new TButtonGadget(LOGON, CM_LOGON));
   toolbar->Insert(
      *new TButtonGadget(LOGOFF, CM_LOGOFF));
  toolbar->Insert(
   *new TSeparatorGadget(16));
  toolbar->Insert(
   *new TButtonGadget(CM_SETUP, CM_LOGSETUP));
  toolbar->Insert(
   *new TButtonGadget(IDB_DIAGNOSTICS, CM_DIAGNOSTICS));
  toolbar->Insert(
   *new TSeparatorGadget(16));
  toolbar->Insert(
   *new TButtonGadget(IDB_INFO, CM_HELPABOUT));
  toolbar->Insert(
   *new TButtonGadget(IDB_FILEEXIT, CMEXIT));
  toolbar->Insert(
   *new TSeparatorGadget(8));

// Enable status line messages when mouse touches a button
  toolbar->SetHintMode(TGadgetWindow::EnterHints);
   
// Insert the toolbar into the frame window
  frame->Insert(*toolbar, TDecoratedFrame::Top);

// Create status line with function keys legend
  statusLine = new TStatusBar(frame, TGadget::Recessed,
    TStatusBar::CapsLock | TStatusBar::NumLock |
    TStatusBar::ScrollLock | TStatusBar::Overtype);
  frame->Insert(*statusLine, TDecoratedFrame::Bottom);

// Assign the decorated MDI frame as the program's main window
  SetMainWindow(frame);
  MainWindow->SetIcon(this, A_N_T_ICON);
  strcpy(filen, "");
}

bool TMicroLanApp::CanClose()
{
   if(timer_set_flag)           //make sure timers were killed
      cp->KillTimer(NULL);
   if(ID_MYTIMER != NULL)
   {
      ::KillTimer(NULL, ID_MYTIMER);
      timecount = 0;
      cp->SaveFile();
   }
   deltree(root);
   MessageBeep(1);
   return TRUE;
}

#pragma argsused

// =============================================================================
// OwlMain
// =============================================================================
int OwlMain(int argc, char* argv[])
{
   if(argc > 1)
   {
      comport = atoi(argv[1]);
      if(comport < 1 || comport > 16)
         comport = 0;    
   }
   else
      comport = 0;
   TMicroLanApp app("Window/Door Monitor with DS2401");
   return app.Run();
}

