/******************************************************************************\
*  Copyright (C) 1995 Dallas Semiconductor Corporation.                        *
*  All rights Reserved. Printed in U.S.A.                                      *
*  This software is protected by copyright laws of                             *
*  the United States and of foreign countries.                                 *
*  This material may also be protected by patent laws of the United States     *
*  and of foreign countries.                                                   *
*  This software is furnished under a license agreement and/or a               *
*  nondisclosure agreement and may only be used or copied in accordance        *
*  with the terms of those agreements.                                         *
*  The mere transfer of this software does not imply any licenses              *
*  of trade secrets, proprietary technology, copyrights, patents,              *
*  trademarks, maskwork rights, or any other form of intellectual              *
*  property whatsoever. Dallas Semiconductor retains all ownership rights.     *
*                                                                              *
*  Compiler: Borland C++ 4.5  OWL                                              *
*  Ver 1.0                                                                     *
*  Date: 4/27/95                                                               *
*                                                                              *
\******************************************************************************/

//==============================================================================
// ftoa - converts float to alpha
//==============================================================================
char *ftoa(double d, char *buff, int numdecplaces)
{
  int i;
  long ltemp;
  char tempbuff[80];
  double mult;


  buff[0] = '\0';
  mult = 100.00;
  ltemp = (long)(mult*d);
// get integer part
  strcpy(buff,ltoa(ltemp/ (long) mult,tempbuff,10));
  strcat(buff,".");
// get decimal part
  ltemp = labs(ltemp);
  ltemp %= (long)mult;

  for (i=0;i<(numdecplaces-1);i++)
  {
    if (ltemp < pow10(i+1))
      strcat (buff,"0");
  }
  strcat(buff,ltoa(ltemp,tempbuff,10));
  return buff;
}

//==============================================================================
// mstrcpy - works with any memory model
//==============================================================================
void mstrcpy(char far *dest, char far *src)
{
   int i = 0;

   do
   {
      dest[i] = src[i];
   }while (src[i++] != 0);
}

//==============================================================================
// memcpy that works with any memory model
//==============================================================================
void mmemcpy(uchar far *dest, uchar far *src, uchar transnum)
{
   uchar jll;

  if (src == NULL)
    dest = NULL;
  else
    for (jll = 0; jll < transnum; jll++)
      *(dest + jll) = *(src + jll);
}

//==============================================================================
// memcmp that works with any memory model
//==============================================================================
int mmemcmp(uchar far *m1, uchar far *m2, int n)
{
  int i;

  for(i=0;i<n;i++)
  {
    if(*(m1 + i) == *(m2 + i))
      continue;
      
    if(*(m1 + i) < *(m2 + i))
      return -(++i); // return negative of index+1 at which failure occured

    return ++i; // return index+1 at which failure occured
  }
  return 0;
}

//==============================================================================
// addtree - recursive - adds a node to the binary tree
//==============================================================================
struct tnode *addtree(struct tnode far *p, int rom1[8])
{
   int cond;

   if (p == NULL)
   {
      if((p = talloc()) != NULL) //initialize node
      {
         //copy the device data to the node
         mmemcpy((uchar far *) p->ROMID, (uchar far *) rom1, 8*sizeof(int));
         mstrcpy(p->word, new_name);
         p->ageflag = age_flag;
         p->count = num_devices;
         p->left = p->right = NULL;
      }
   }
   else if((cond = mmemcmp((uchar far *) rom1, (uchar far *)p->ROMID, 8 * sizeof(int)/*(uchar far *) p + 25, 8*sizeof(int)*/)) == 0)
      p->count++;
   else if (cond < 0)   //add to left node
      p->left = addtree(p->left, rom1);
   else                 //add to right node
      p->right = addtree(p->right, rom1);
   return p;
}

//==============================================================================
// deltree - deletes the tree when finished with it - recursive
//==============================================================================
void deltree(struct tnode *p)
{
   if(p == NULL) return;        //steps through each node and deletes it

   struct tnode far *p1;
   
   if(p->left != NULL)
   {
      p1 = p->left;
      p->left = NULL;
      deltree(p1);
   }

   if(p->right != NULL)
   {
      p1 = p->right;
      p->right = NULL;
      deltree(p1);
   }

   delete(p);
   return;
}

//==============================================================================
// agetree - determine if device is missing or not
//==============================================================================
int agetree(struct tnode *p)
{
   if (p != NULL)
   {
      agetree(p->left);
      if(!p->ageflag && !switchflag)
      {  
         szmissing++;
         strcpy(p->present, "OPEN");
      }
      else if(p->ageflag && switchflag)
      {
         szmissing++;
         strcpy(p->present, "OPEN");
      }
      else
      {
         strcpy(p->present, "CLOSED");
      }
      if(!switchflag)
         p->ageflag = 0;
      else if(switchflag)
         p->ageflag = 1;
      agetree(p->right);  //age the tree
   }
   if(szmissing != 0)    //return false if any devices missing
      return 0;   
   return 1;   
}

//==============================================================================
// agetreenull - for the case where device missing means system ok
//==============================================================================
int agetreenull(struct tnode *p)
{
   if (p != NULL)
   {
      agetreenull(p->left);
      p->ageflag = 0;       //set the ageflag to 0 for each node
      agetreenull(p->right);
   }
   return TRUE;
}
//==============================================================================
// fill the listbox array with device info - recursive
//==============================================================================
int fill_array_with_info(struct tnode *p)
{
   if(p!= NULL)
   {
      fill_array_with_info(p->left);
      strcpy(device_array[p->count], p->word);  //copy the device name
      strcpy(device_array[p->count+20], p->present);  //copy device status
      fill_array_with_info(p->right);
   }
   return TRUE;  
}

//==============================================================================
// search the tree for a romid - recursive
//==============================================================================
struct tnode *search_tree(struct tnode *p, int rom1[8])
{
   if(!p) return p;
   int cond;
   //compare the romid
   while ((cond = mmemcmp((uchar *) p->ROMID, (uchar *) rom1, 16)) != 0)
   {
      if(cond > 0) p = p->left; //its to the left
      else p = p->right;        //its to the right
      if(p == NULL)
         break;
   }
   if(cond == 0 && p != NULL && !switchflag)
      p->ageflag = 1;                         //found
   else if(cond == 0 && p != NULL && switchflag)
      p->ageflag = 1;                         //found
   return p;                                  
}
//==============================================================================
// initializes the structures
//==============================================================================
struct tnode far *talloc()
{
   return new struct tnode;
}

//==============================================================================
// checks the tree for missing devices
//==============================================================================
int Check_Tree()
{
//   struct tnode far *searchroot; 
   rom[0] = 0x00;
   if(!TMValidSession(handle))
   {
      handle = TMStartSession(comport);
      TMSetup(handle);
   }
   if(!switchflag)
   {
      if(TMFirst(handle, GB))
      {
         TMRom(handle, GB, rom);
         search_tree(root, rom);
      }
      while(TMNext(handle, GB))
      {
         rom[0] = 0x00;
         TMRom(handle, GB, rom);
         search_tree(root, rom);
      }   
   
      return(agetree(root));
   }
   else
   {
      agetreenull(root);
      if(TMFirst(handle, GB))
      {
         TMRom(handle, GB, rom);
         search_tree(root, rom);
      }
      else
         return (agetree(root));
      while(TMNext(handle, GB))
      {
         rom[0] = 0x00;
         TMRom(handle, GB, rom);
         search_tree(root, rom);
      }   
   
      return(agetree(root));
   }   
}

//==============================================================================
// Read the previous sessions binary tree - recursive
//==============================================================================
void read_settings1(void)
{
   struct tnode *p;
   p = new tnode;
   if(in_stream == NULL)
      return;
   while((read(in_stream, (void *) p, sizeof(tnode))) == sizeof(tnode))
   {
      if(root == NULL)
      {
         mstrcpy(new_name, p->word);
         age_flag = p->ageflag;
         p->count = num_devices;

         root = addtree(root, p->ROMID);
         num_devices++;
      }
      else
      {
         mstrcpy(new_name, p->word);
         age_flag = p->ageflag;
         addtree(root, p->ROMID);
         num_devices++;
      }
   }
   close(in_stream);
   return;
}

//==============================================================================
// Read the previous sessions settings
//==============================================================================
void read_settings(void)
{
   read(in_stream, (void *)&comport, sizeof(comport));
   read(in_stream, (void *)&switchflag, sizeof(switchflag));
   read(in_stream, (void *)&log_pass_flag, sizeof(log_pass_flag));
   read(in_stream, (void *)&global_time_interval, sizeof(global_time_interval));
   read(in_stream, (void *)filen, 80);
   read(in_stream, (void *)&log_file_open_flag, sizeof(log_file_open_flag));

   read_settings1();
   read(in_stream, (void *)&num_devices, sizeof(num_devices));
}

//==============================================================================
// Save the current sessions binary tree
//==============================================================================
void save_setup1(struct tnode *p)
{
   if (p!=NULL)
   {
      save_setup1(p->left);
         write(out_stream, (void *) p, sizeof(tnode));
      save_setup1(p->right);
   }
   return;
}

//==============================================================================
// Save the current sessions settings
//==============================================================================
void save_setup(struct tnode *p)
{
   write(out_stream, (void *)&comport, sizeof(comport));
   write(out_stream, (void *)&switchflag, sizeof(switchflag));
   write(out_stream, (void *)&log_pass_flag, sizeof(log_pass_flag));
   write(out_stream, (void *)&global_time_interval, sizeof(global_time_interval));
   write(out_stream, (void *)filen, 80);
   write(out_stream, (void *)&log_file_open_flag, sizeof(log_file_open_flag));

   save_setup1(p);
   write(out_stream, (void *)&num_devices, sizeof(num_devices));
}

