/******************************************************************************\
*  Copyright (C) 1995 Dallas Semiconductor Corporation.                        *
*  All rights Reserved. Printed in U.S.A.                                      *
*  This software is protected by copyright laws of                             *
*  the United States and of foreign countries.                                 *
*  This material may also be protected by patent laws of the United States     *
*  and of foreign countries.                                                   *
*  This software is furnished under a license agreement and/or a               *
*  nondisclosure agreement and may only be used or copied in accordance        *
*  with the terms of those agreements.                                         *
*  The mere transfer of this software does not imply any licenses              *
*  of trade secrets, proprietary technology, copyrights, patents,              *
*  trademarks, maskwork rights, or any other form of intellectual              *
*  property whatsoever. Dallas Semiconductor retains all ownership rights.     *
*                                                                              *
*  Compiler: Borland C++ 4.5  OWL                                              *
*  Ver 1.0                                                                     *
*  Date: 4/27/95                                                               *
*                                                                              *
\******************************************************************************/
//==============================================================================
// ftoa - convert float to string
//==============================================================================
char *ftoa(double d, char *buff, int numdecplaces)
{
  int i;
  long ltemp;
  char tempbuff[80];
  double mult;


  buff[0] = '\0';
  mult = 100.00;
  ltemp = (long)(mult*d);
// get integer part
  strcpy(buff,ltoa(ltemp/ (long) mult,tempbuff,10));
  strcat(buff,".");
// get decimal part
  ltemp = labs(ltemp);
  ltemp %= (long)mult;

  for (i=0;i<(numdecplaces-1);i++)
  {
    if (ltemp < pow10(i+1))
      strcat (buff,"0");
  }
  strcat(buff,ltoa(ltemp,tempbuff,10));
  return buff;
}

//==============================================================================
// strcpy for any memory model
//==============================================================================
void mstrcpy(char far *dest, char far *src)
{
   int i = 0;

   do
   {
      dest[i] = src[i];
   }while (src[i++] != 0);
}
//==============================================================================
// memcpy for any memory model
//==============================================================================
void mmemcpy(uchar far *dest, uchar far *src, uchar transnum)
{
   uchar jll;

  if (src == NULL)
    dest = NULL;
  else
    for (jll = 0; jll < transnum; jll++)
      *(dest + jll) = *(src + jll);
}
//==============================================================================
// memcpy for any memory model
//==============================================================================
int mmemcmp(uchar far *m1, uchar far *m2, int n)
{
  int i;

  for(i=0;i<n;i++)
  {
    if(*(m1 + i) == *(m2 + i))
      continue;
      
    if(*(m1 + i) < *(m2 + i))
      return -(++i); // return negative of index+1 at which failure occured

    return ++i; // return index+1 at which failure occured
  }
  return 0;
}

//==============================================================================
// ReadSwitchState - Gets the state of the DS2405 switch - recursive
//==============================================================================
void ReadSwitchState(struct tnode *p)
{
   int i;
   if(p == NULL)
      return;

   if(!TMValidSession(handle))     //check session
   {
      handle = TMStartSession(comport);
      TMSetup(handle);
      TMTouchReset(handle);
   }

   if (p != NULL)
   {
      ReadSwitchState(p->left);      //read the left node
      TMRom(handle, GB, p->ROMID);    //access the device
      TMStrongAccess(handle, GB);    //do a strong access to get the state
      i = TMTouchByte(handle, 0xFF); //returns the state in i
      
      if(!p->ageflag)
         i = 5; //dummy statement to keep it from writing over missing flag
      
      else if(!switchflag && i)
         p->ageflag = 2;        //device open
     
      else if(!switchflag && !i)
         p->ageflag = 1;        //device closed
     
      else if(switchflag && i)
         p->ageflag = 2;        //device closed
      
      else
         p->ageflag = 1;        //device open

      ReadSwitchState(p->right);  //read the right node
   }  
   return;
}

//==============================================================================
// SetSwitchState - sets the state of the switch to 0
//==============================================================================
void SetSwitchState()
{
   if(!TMValidSession(handle))     //check session
   {
      handle = TMStartSession(comport);
      TMSetup(handle);
      TMTouchReset(handle);
   }
   if(TMFirst(handle, GB))       //access each 2405 and turn off the switch
   {
      rom[0] = 0x00;
      TMRom(handle, GB, rom);
      SwitchOff((uchar *)rom);
   }
   while(TMNext(handle, GB))
   {
      rom[0] = 0x00;
      TMRom(handle, GB, rom);
      SwitchOff((uchar *)rom);
   }
   ReadSwitchState(root);  //find out if switch is still high or low
                           
}

//==============================================================================
// addtree - add a node to the binary tree
//==============================================================================
struct tnode *addtree(struct tnode far *p, int rom1[8])
{
   int cond;

   if (p == NULL)
   {
      if((p = talloc()) != NULL)  //initialize structure
      {
         //write device info in struct
         mmemcpy((uchar far *) p->ROMID, (uchar far *) rom1, 8*sizeof(int));
         mstrcpy(p->word, new_name);
         p->ageflag = age_flag;
         p->count = num_devices;
         p->left = p->right = NULL;
      }
      
   }
   else if((cond = mmemcmp((uchar far *) rom1, (uchar far *) p->ROMID, 8*sizeof(int))) == 0)
      p->count++;
   else if (cond < 0)
      p->left = addtree(p->left, rom1); //add it on left node
   else
      p->right = addtree(p->right, rom1);//add it on right node
   return p;
}

//==============================================================================
// deltree - delete the binary tree
//==============================================================================
void deltree(struct tnode *p)
{
   
   if(p == NULL) return;

   struct tnode far *p1;           //delete the tree node by node
   
   if(p->left != NULL)
   {
      p1 = p->left;
      p->left = NULL;
      deltree(p1);
   }

   if(p->right != NULL)
   {
      p1 = p->right;
      p->right = NULL;
      deltree(p1);
   }

   delete(p);
   return;
}

//==============================================================================
// agetree - is device open or closed - recursive
//==============================================================================
int agetree(struct tnode *p)
{
   if (p != NULL)
   {
      agetree(p->left);       //do left node

      if(p->ageflag == 2 && !switchflag)
      {
         szmissing++;
         strcpy(p->present, "OPEN");
      }
      else if(p->ageflag == 2 && switchflag)
      {
         szmissing++;
         strcpy(p->present, "CLOSED");
      }
      else if(p->ageflag == 1 && !switchflag)
         strcpy(p->present, "CLOSED");

      else if(p->ageflag == 1 && switchflag)
         strcpy(p->present, "OPEN");

      else if(!p->ageflag)
         strcpy(p->present, "MISSING");

      if(!switchflag)
         p->ageflag = 0;

      else if(switchflag)
         p->ageflag = 1;

      agetree(p->right);           //do right node
   }
   if(szmissing != 0)
      return 0;   
   return 1;   
}

//==============================================================================
// agetreenull - reset the ageflag to 0 - recursive
//==============================================================================
int agetreenull(struct tnode *p)
{
   if (p != NULL)
   {
      agetreenull(p->left);
      p->ageflag = 0;
      agetreenull(p->right);
   }
   return TRUE;
}

//==============================================================================
// search the tree for a ROMID - recursive
//==============================================================================
struct tnode *search_tree(struct tnode *p, int rom1[8])
{
   if(!p) return p;
   int cond;
   while ((cond = mmemcmp((uchar *) p->ROMID, (uchar *) rom1, 16)) != 0)
   {
      if(cond > 0) 
         p = p->left;
      else 
         p = p->right;
      if(p == NULL)
         break;
   }
   if(cond == 0 && p != NULL && !switchflag)
      p->ageflag = 1;
   else if(cond == 0 && p != NULL && switchflag)
      p->ageflag = 1;
   return p;
}

//==============================================================================
// fill the listbox array with info
//==============================================================================
int fill_array_with_info(struct tnode *p)
{
   if(p!= NULL)
   {
      fill_array_with_info(p->left);
      strcpy(device_array[p->count], p->word); //copy device name
      strcpy(device_array[p->count+20], p->present); //open | closed
      fill_array_with_info(p->right);
   }
   return TRUE;  
}

//==============================================================================
// initialize the structure
//==============================================================================
struct tnode far *talloc()
{
   return new struct tnode;
}

//==============================================================================
//  check for each device being present on the microlan 
//==============================================================================
int Check_Tree()
{
//   struct tnode far *searchroot; 
   rom[0] = 0x00;
   if(!TMValidSession(handle))            //check the session
   {
      handle = TMStartSession(comport);
      TMSetup(handle);
   }
   if(!switchflag)
   {
      if(TMFirst(handle, GB))          //find all devices and set ageflag
      {
         TMRom(handle, GB, rom);
         search_tree(root, rom);
      }
      while(TMNext(handle, GB))
      {
         rom[0] = 0x00;
         TMRom(handle, GB, rom);
         search_tree(root, rom);
      }   
      SetSwitchState();             //set each device to off

      return(agetree(root));
   }
   else
   {
      agetreenull(root);
      if(TMFirst(handle, GB))
      {
         TMRom(handle, GB, rom);
         search_tree(root, rom);
      }
      else
         return (agetree(root));
      while(TMNext(handle, GB))
      {
         rom[0] = 0x00;
         TMRom(handle, GB, rom);
         search_tree(root, rom);
      }   
      SetSwitchState();             //set each device to off
      return(agetree(root));
   }   
}

//==============================================================================
// read previous sessions binary tree
//==============================================================================
void read_settings1(void)
{
   struct tnode *p;
   p = new tnode;
   if(in_stream == NULL)
      return;
   while((read(in_stream, (void *) p, sizeof(tnode))) == sizeof(tnode))
   {
      if(root == NULL)
      {
         mstrcpy(new_name, p->word);
         age_flag = p->ageflag;
         root = addtree(root, p->ROMID);
         num_devices++;
      }
      else
      {
         mstrcpy(new_name, p->word);
         age_flag = p->ageflag;
         addtree(root, p->ROMID);
         num_devices++;
      }
   }
   close(in_stream);
   return;
}

//==============================================================================
// read previous sessions settings 
//==============================================================================
void read_settings(void)
{
   read(in_stream, (void *)&comport, sizeof(comport));
   read(in_stream, (void *)&switchflag, sizeof(switchflag));
   read(in_stream, (void *)&log_pass_flag, sizeof(log_pass_flag));
   read(in_stream, (void *)&global_time_interval, sizeof(global_time_interval));
   read(in_stream, (void *)filen, 80);
   read(in_stream, (void *)&log_file_open_flag, sizeof(int));
   read_settings1();
   read(in_stream, (void *)&num_devices, sizeof(num_devices));
}

//==============================================================================
// save the previous sessions binary tree 
//==============================================================================
void save_setup1(struct tnode *p)
{
   if (p!=NULL)
   {
      save_setup1(p->left);
         write(out_stream, (void *) p, sizeof(tnode));
      save_setup1(p->right);
   }
   return;
}

//==============================================================================
// save the previous sessions settings
//==============================================================================
void save_setup(struct tnode *p)
{
   write(out_stream, (void *)&comport, sizeof(comport));
   write(out_stream, (void *)&switchflag, sizeof(switchflag));
   write(out_stream, (void *)&log_pass_flag, sizeof(log_pass_flag));
   write(out_stream, (void *)&global_time_interval, sizeof(global_time_interval));
   write(out_stream, (void *)filen, 80);
   write(out_stream, (void *)&log_file_open_flag, sizeof(int));
   save_setup1(p);
   write(out_stream, (void *) &num_devices, sizeof(num_devices));
}


//==============================================================================
// checks the switch state of the 2405 
//==============================================================================
uchar SwitchState(uchar far *RI)
{
   int i;
   if(!TMValidSession(handle))
   {
      TMEndSession(handle);
      handle = TMStartSession(comport);
      TMSetup(handle);
   }

   TMRom(handle, GB, (int *)RI);
   i = TMStrongAlarmAccess(handle, GB);
   return i;
}

//==============================================================================
// Turn the DS2405 switch on
//==============================================================================
void SwitchOn(uchar far *RI)
{
   if(!TMValidSession(handle)) //check the session
   {
      handle = TMStartSession(comport);
      TMSetup(handle);
   }

   if(!SwitchState(RI))  //if device is off turn it on
   {
      
      TMAccess(handle, GB);  //turn device on
      wait_counts(2400L);    //wait 20 ms
   }
   return;
}

//==============================================================================
// turn the ds2405 switch off
//==============================================================================
void SwitchOff(uchar far *RI)
{

   if(!TMValidSession(handle)) //check session
   {
      handle = TMStartSession(comport);
      TMSetup(handle);
   }

   if(SwitchState(RI))     //if device is on turn it off
   {
      TMAccess(handle, GB);
   }
   return;
}

//==============================================================================
// wait_counts - 8250 counter
//==============================================================================
static void wait_counts(ulong wait_count)
{
   unsigned short icount, ccount = 0, lcount;

   outp(0x43, 0);                              // Freeze the initial count 
   lcount = inp(0x40);                           // Read low byte of count 
   icount = (inp(0x40) << 8) | lcount;           // Read the initial count 
   do
   {
      outp(0x43, 0);                           // Freeze the current count 
      lcount = inp(0x40);                        // Read low byte of count 
      ccount = (inp(0x40) << 8) | lcount;        // Read the initial count 
   }                                                                       
   while ((unsigned short) (icount - ccount) < wait_count);          // Burn cycles
   
}
