/******************************************************************************\
*  Copyright (C) 1995 Dallas Semiconductor Corporation.                        *
*  All rights Reserved. Printed in U.S.A.                                      *
*  This software is protected by copyright laws of                             *
*  the United States and of foreign countries.                                 *
*  This material may also be protected by patent laws of the United States     *
*  and of foreign countries.                                                   *
*  This software is furnished under a license agreement and/or a               *
*  nondisclosure agreement and may only be used or copied in accordance        *
*  with the terms of those agreements.                                         *
*  The mere transfer of this software does not imply any licenses              *
*  of trade secrets, proprietary technology, copyrights, patents,              *
*  trademarks, maskwork rights, or any other form of intellectual              *
*  property whatsoever. Dallas Semiconductor retains all ownership rights.     *
\******************************************************************************/


// ===========================================================
// The MDI child window class
// ===========================================================
class TDecoChild: public TMDIChild {
public:
   time_t timer;   //used to get system time
   struct tm *tblock;   
   TDecoChild(TMDIClient& parent, const char far *name); //constructor
   ~TDecoChild();    //destructor
   void SetupWindow(); //setup window function
   int SaveFile();  //function to save the file
   void EvTimer(UINT timerId);  //timer to read the microlan
   string filename;  //name of the file to save to
   int numRows;
private:
   TListBox* listbox;  //multicolumn listbox
   TStatic* title1;    //static text variables
   TStatic* title2;
   TStatic* time_text;
   TIArrayAsVector<string> sArray; //array for devices.
//   TIArrayAsVector<string> s1Array;//array for temperatures.
   DECLARE_RESPONSE_TABLE(TDecoChild);
};

DEFINE_RESPONSE_TABLE1(TDecoChild, TMDIChild)
   EV_WM_TIMER,
END_RESPONSE_TABLE;
//==============================================================================
// Constructor
//==============================================================================
TDecoChild::TDecoChild(TMDIClient& parent, const char far *name)
  : TMDIChild(parent, name), sArray(SARRAY_HIGH, SARRAY_LOW, SARRAY_DELTA)
//  s1Array(SARRAY_HIGH, SARRAY_LOW, SARRAY_DELTA)
{
   int i;
   filename = name;   //copy filename
   //setup child window attributes
   Attr.Style |= MDIS_ALLCHILDSTYLES;
   Attr.Style |=WS_MAXIMIZE;
   Attr.Style |= WS_VSCROLL;
   Attr.Style |= WS_HSCROLL;
   //initialize scroller
   //Scroller = new TScroller(this, 0, 0, 0, 0);
   numRows = 0;
   
   TModule *TM;     //needed for initialing the timer
   TM = GetModule();

   // timer for saving file every few minutes.  time controlled by a define statement;
   if(strcmp(name, ""))
      ID_MYTIMER = ::SetTimer(NULL, 1, 10000, (TIMERPROC)MakeProcInstance((FARPROC)Save, TM->GetInstance())); 
   for(i = 0; i < 60; i++)
      device_array[i][0] = 0;
}
//==============================================================================
// Destructor - clean up all variables
//==============================================================================
TDecoChild::~TDecoChild()
{
   //delete all necessary variables and set them to null
   if(listbox)
    delete listbox;
   listbox = 0;
   if(title1)
      delete title1;
   title1 = 0;
   if(title2)
      delete title2;
   title2 = 0;
   if(time_text)
      delete time_text;
   time_text = 0;
      
}
//==============================================================================
// SetupWindow Function, Initialize all static text variables
//==============================================================================
void TDecoChild::SetupWindow()
{
   TEXTMETRIC tm;
     
   dc = new TClientDC(*this);
   dc->GetTextMetrics(tm);
   title1 = new TStatic(this, -1, "Name",50, 25, 50, 20);
   title2 = new TStatic(this, -1, "Temperature at", 250, 25, 100, 20);
   time_text = new TStatic(this, -1, "", 375, 25 , 200, 20);
   int cyChar = tm.tmHeight +tm.tmExternalLeading;
   listbox = new TListBox(this, ID_TEMPLISTBOX, 50, 50, 600, cyChar*20);   
   listbox->Attr.Style |= LBS_MULTICOLUMN;
   listbox->Attr.Style &= ~WS_HSCROLL;
   delete dc;
   
   int tab_pos[3];
   tab_pos[0] = 0;
   tab_pos[1] = 200;
   tab_pos[2] = 400;
   TFrameWindow::SetupWindow();

   ID_EVTIMER = SetTimer(1, global_time_interval*1000);
   timer_set_flag = TRUE;
   //EvTimer(NULL);
   if(!TMValidSession(handle))
   {
      TMEndSession(handle);
      handle = TMStartSession(comport);
      TMSetup(handle);
   }
   TMTouchReset(handle);
   int j = TMFirst(handle, GB); 
   if(j == 1)
   {
      rom[0] = 0x00;
      TMRom(handle, GB, rom);
      TMTouchByte(handle, 0x55);
      TMRom(handle, GB, rom);
      TMAccess(handle, GB);
      TMTouchByte(handle, 0x44);
      
      finished_flag = FALSE;
      TModule *TM;
      TM = GetModule();
      // timer for saving file every few minutes.  time controlled by a define statement;
      ID_TEMPTIMER = ::SetTimer(NULL, 2, 1000, (TIMERPROC)MakeProcInstance((FARPROC)Temperature, TM->GetInstance()));
   }

   listbox->SetColumnWidth(200);
   listbox->SetTabStops(2, tab_pos);
}
//==============================================================================
// Save - Not part of the child class, necessary to use a windows timer
//==============================================================================
WORD FAR PASCAL Save(HWND, WORD, WORD, LONG)
{
   cp->SaveFile(); //call class save function
   return 0;
}
//==============================================================================
// SaveFile - saves the log to a file
//==============================================================================
int TDecoChild::SaveFile()
{
   char c[80]; //character string to write to file
   const char* cst; //const string to get info from sArray
   int tempNumRows, len;

   if(timecount == 0)
   {
      //check to make sure file needs to be saved.
      if((tempNumRows = sArray.GetItemsInContainer()) > 0)
      {
         FILE *fp; //file pointer
         fp = fopen(filename.c_str(), "a+");  //open the file
         if(fp != NULL)              //make sure the file opened
         {
            for(int i = 0; i < tempNumRows; i++)  //save anything necessary 
            {   
               cst = sArray[i]->c_str(); //get the current row from the array
               strcpy(c, cst);      //copy the const string to c to modify it
               c[(len = strlen(c))] = 0x0a;    //add the windows end of line character
               c[len+1] = 0;              //terminate with a zero
               fwrite(c, strlen(c), 1, fp);   //write to the file
            }
            sArray.Flush();    //clear the save array
            numRows = 0;     //clear the counter
            timecount = WAIT_TIME; //reset the save variable
            fclose(fp);        //close the file
         }
         else        //couldn't open the specified file
         {
            MessageBox("Failed to open the log file", "ERROR", MB_ICONEXCLAMATION+MB_OK);
         }
      }
      else   //nothing to save
         timecount = WAIT_TIME;
   }
   else   //not time to save yet
      --timecount;
   return TRUE;
}
//==============================================================================
// EvTimer function - checks status of microlan and prints the info
//==============================================================================
void TDecoChild::EvTimer(UINT)
{
   char s[200];  //string to add to sArray, not compatible with save function
   int i , j;
   if(!finished_flag) //make sure not taking temperature
      return;
   if(kill_timer_flag) //kill the temperature timer each pass
   {
      KillTimer(NULL);
      ::KillTimer(NULL, ID_MYTIMER);
      return;
   }
   if(num_waits == 0) //see if it has been specified time between passes
   {
      szmissing[0] = 0; //clear the missing device string
      if(Check_Tree())  //check the microlan
      {
         TModule *TM;     //needed for the timers
         TM = GetModule();
         if(!TMValidSession(handle)) //check for a valid session
         {
            handle = TMStartSession(comport);
            TMSetup(handle);
         }
         i = TMTouchReset(handle);
         j = TMFirst(handle, GB);  //find the first 1820/1920
         if(j == 1)
         {
            rom[0] = 0x00;                //clear the lsb of the rom data
            TMRom(handle, GB, rom);       //get the romid
            TMTouchByte(handle, 0x55);    //send the skip rom command
                                          //might not work with several devices
                                          //on the comport
            TMRom(handle, GB, rom);       //access the first 1820/1920
            TMAccess(handle, GB);
            TMTouchByte(handle, 0x44);    //send convert T command
            
            finished_flag = FALSE;        //set the temperature variable
            
            // timer for saving file every few minutes.  time controlled by a define statement;
            ID_TEMPTIMER = ::SetTimer(NULL, 2, 1000, (TIMERPROC)MakeProcInstance((FARPROC)Temperature, TM->GetInstance()));
         }
   
         //determine which logging procedure is in use.
         //log changed passes, log all passes or alarming devices
         if(!log_pass_flag  || (log_pass_flag && alarm_flag))
         {
            
            sz[0] = 0;
            timer = time(NULL);  //get the time from the system
            tblock = localtime(&timer);
            strcpy(s, asctime(tblock));
            s[strlen(s) - 1] = 0;
              sArray.Add(new string(s)); //add to the array
            numRows++;
            time_text->SetText(s);  //set the time static text
            
            for(i = 0; i < 60; i++)        //clear the device array
               strcpy(device_array[i], "");
            listbox->ClearList();       //clear the listbox
            fill_array_with_info(root); //refil the array with device info
            for(i = 0; i < 60; i++)     //insert the strings
               listbox->InsertString(device_array[i], i);

            for(i = 0; i < num_devices; i++) //print out the temperature of each device
            {
               numRows++;               //increment the counter for each line
               strcpy(s, device_array[i]); //copy the proper entry to the string
               for(j = strlen(s); j < 25; j++)
                  s[j] = ' ';
               s[25] = 0;           //left justify each line
               strcat(s, device_array[i+20]);    //add the temperature
               for(j = strlen(s); j < 40; j++)
                  s[j] = ' ';
               s[40] = 0;
               strcat(s, device_array[i+40]);  //alarming or not?
               sArray.Add(new string(s));     //add the string
            }
            strcpy(s, " ");      //add a space
            sArray.Add(new string(s));
            numRows++;   
               
            last_pass = FALSE;      //last pass = FALSE for changed passes only
            alarm_flag = FALSE;     //nothing alarming
         }
//         else if((log_pass_flag == 1 && !alarm_flag))
//         {
//            MessageBeep(1);
//            timer = time(NULL);   //get the time
//            tblock = localtime(&timer);
//            strcpy(s, asctime(tblock));
//            s[strlen(s) - 1] = 0;
//            time_text->SetText(s);   //set the static text
//            for(i = 0; i < 60; i++)  //insert it to the listbox
//               strcpy(device_array[i], "");
//            fill_array_with_info(root);
//            listbox->ClearList();
//            for(i = 0; i < 60; i++)
//               listbox->InsertString(device_array[i], i);   
//         } 
      }
      else   //device missing, but still attempt to take the temperature
      {
         TModule *TM;      //for timer
         TM = GetModule();
         if(!TMValidSession(handle))  //check the session handle
         {
            handle = TMStartSession(comport);
            TMSetup(handle);
         }
         i = TMTouchReset(handle);
         j = TMFirst(handle, GB); 
         if(j == 1)
         {                        //broadcast the convert T command
            rom[0] = 0x00;
            TMRom(handle, GB, rom);
            TMTouchByte(handle, 0x55);
            TMRom(handle, GB, rom);
            TMAccess(handle, GB);
            TMTouchByte(handle, 0x44);
            
            finished_flag = FALSE;
            // timer for saving file every few minutes.  time controlled by a define statement;
            ID_TEMPTIMER = ::SetTimer(NULL, 2, 1000, (TIMERPROC)MakeProcInstance((FARPROC)Temperature, TM->GetInstance()));
            finished_flag = FALSE;
         }
         //determing which logging procedure is in use;
         if(!log_pass_flag || (log_pass_flag && alarm_flag))
         {
            timer = time(NULL);
            tblock = localtime(&timer);
            strcpy(s, asctime(tblock));
            s[strlen(s) - 1] = 0;
            time_text->SetText(s);
            numRows++;
            sArray.Add(new string(s));
            for( i = 0; i < 60; i++)
               strcpy(device_array[i], "");
            listbox->ClearList();
            fill_array_with_info(root);
                     
            for(i = 0; i < num_devices; i++)
            {
               numRows++;
               strcpy(s, device_array[i]);
               for(int j = strlen(s); j < 25; j++)
                  s[j] = ' ';
               s[25] = 0;
               strcat(s, device_array[i+20]);
               for(j = strlen(s); j < 40; j++)
                  s[j] = ' ';
               s[40] = 0;
               strcat(s, device_array[i+40]);
               sArray.Add(new string(s));
            }
            strcpy(s, " ");
            numRows++;
            sArray.Add(new string(s));

            for(i = 0; i < 60; i++)
               listbox->InsertString(device_array[i], i);
            
            last_pass = TRUE;
         }
      }
      num_waits = old_num_waits;  //reset the count down timer
   }
   else  //hasn't been long enough
      num_waits--;
}

//==============================================================================
//ev_timer function used to measure temperature of all devices on the one-wire;
//while this function is in use, the TDecoChild::EvTimer function cannot take place;
//==============================================================================
WORD FAR PASCAL Temperature(HWND, WORD, WORD, LONG)
{
   double temp;      //variable for the temperature
   char buf[10], buffer[60];
   int scratchpad[8], next_device = FALSE; //scratchpad from the 1820
   tnode *struct1;                       //used for searching the binary tree
   int rom2[8];
   if(!TMValidSession(handle)) //check the session
   {
      handle = TMStartSession(comport);
      TMSetup(handle);
   }
   if(!changing_temp)      //make sure not changing the temperatures
   {
         memcpy(rom2, rom, sizeof(rom2));
         TMTouchByte(handle, 0x55);
         TMRom(handle, GB, rom);
         TMAccess(handle, GB);        //access the device
         TMTouchByte(handle, 0xBE);   //get the temperature
         for(int i = 0; i < 9; i++)
            scratchpad[i] = TMTouchByte(handle, 0xFF);
         if(!flag_celsius)         //celsius
         {
            //shift the scratchpad right, lose the lsb
            temp = (double)(scratchpad[0] >> 1); 
            if(scratchpad[0] & 1)     //check lsb to see if its 1/2 degree
               temp = temp + 0.5;
         }
         else          //fahrenheit
         {
            temp = (double)(scratchpad[0] >> 1);
            if(scratchpad[0] & 1)
               temp = temp + 0.5;
            temp = temp * 9.0 / 5.0 + 32.0;   //convert to fahrenheit
         }
         ftoa(temp, buf, 1);     //call my float to alpha function
         if(!flag_celsius)       //append approiate temperature symbol
            strcat(buf, "C");
         else
            strcat(buf, "F");
         struct1 = search_tree(root, rom); //make sure device is included in program                 
         if(scratchpad[1] == 0xFF && struct1 != NULL) //see if below 0
         {
            mstrcpy(struct1->temperature, "-");
            strcat(struct1->temperature, buf);   //copy temperature to the struct
         }
         else if(struct1 != NULL)
            mstrcpy(struct1->temperature, buf);  //copy temperature to the struct
         strcpy(buffer, struct1->word); //copy the name to the string
         strcat(buffer, " ");           //add a space
         strcat(buffer, buf);          //add the temperature
         strcat(sz, buffer);         //copy to global string
         strcat(sz, " ");            //add a space
         //Search for rest of devices and take temperatures;
         while(TMNext(handle, GB)==1)  //convert the rest of the device temperatures
         {
            rom[0] = 0x00;
            TMRom(handle, GB, rom); //access the 1820
            if(memcmp(rom2, rom, sizeof(rom)) != 0) //make sure it's not the previous
            //device           
            {
               struct1 = search_tree(root, rom); //find the device in the b.t.
               if(struct1 != NULL)
               {
                  next_device = TRUE;    //send the convert T command
                  TMTouchByte(handle, 0x55);
                  TMRom(handle, GB, rom);
                  TMAccess(handle, GB);
                  TMTouchByte(handle, 0x44);
                  break;
               }
               next_device = FALSE;
            }
         }
         if(!next_device)    //no more devices
         {
            finished_flag = TRUE;  //set finished flag
            first_time = FALSE;    //kill the temperature timer
            ::KillTimer(NULL, ID_TEMPTIMER);
            if(alarm_on)            //if alarm's are on, do alarm search 
                                    //alarms are always on
               if(alarm_search())
                  sz[0] = 0;    //clear the temperature 
            
         }
      
   }
      return 0;
}


