/******************************************************************************\
*  Copyright (C) 1995 Dallas Semiconductor Corporation.                        *
*  All rights Reserved. Printed in U.S.A.                                      *
*  This software is protected by copyright laws of                             *
*  the United States and of foreign countries.                                 *
*  This material may also be protected by patent laws of the United States     *
*  and of foreign countries.                                                   *
*  This software is furnished under a license agreement and/or a               *
*  nondisclosure agreement and may only be used or copied in accordance        *
*  with the terms of those agreements.                                         *
*  The mere transfer of this software does not imply any licenses              *
*  of trade secrets, proprietary technology, copyrights, patents,              *
*  trademarks, maskwork rights, or any other form of intellectual              *
*  property whatsoever. Dallas Semiconductor retains all ownership rights.     *
\******************************************************************************/

#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <io.h>
#include <fcntl.h>
#include <sys\stat.h>
#include <owl\applicat.h>
#include <owl\dc.h>
#include <owl\dialog.h>
#include <owl\opensave.h>
#include <windows.h>
#include <owl\scroller.h>
#include <classlib\arrays.h>
#include <cstring.h>
#include <owl\gdiobjec.h>
#include <stdio.h>
#include <owl\controlb.h>
#include <owl\gadget.h>
#include <owl\buttonga.h>
#include <owl\messageb.h>
#include <owl\statusba.h>
#include <time.h>
#include <bwcc.h>
#include <dir.h>
#include <owl\decmdifr.h>
#include <owl\mdi.h>
#include <owl\mdichild.h>
#include <mem.h>
#include <owl\scrollba.h>
#include <owl\combobox.h>
#include <owl\static.h>
#pragma  hdrstop

#define MAXTIMEINTERVAL 60.0      //maximum time between microlan logging
#define MINTIMEINTERVAL 0.0       //min time interval
//#define MAX_LINES 48

#define SARRAY_HIGH 249           //size of the array to save the file info to
#define SARRAY_LOW 0
#define SARRAY_DELTA 125          //expandable, not necessary with 3 minute saving

#define TICK_SEG 0x40             //used in wait_counts function
#define TICK_OFF 0x6c

#define TRUE 1
#define FALSE 0
//#define LINE1 "Temperature Monitor with 1820"
//#define LINE2 ""
//#define LINE3 "Dallas Semiconductor"
//#define LINE4 "Copyright 1995"
#define WAIT_TIME 10  //amount of time to wait to save the file, 10*30seconds

#include "define3.h"
#include "exp3.h"
#include "tree3.cpp"
#include "dialog3.cpp"
#include "child3.cpp"



// =============================================================================
// MDI client class
// =============================================================================

class TMicroLanClient: public TMDIClient {
public:
   
   TMicroLanClient(): TMDIClient() {};   //constructor
   TOpenSaveDialog::TData *data;  //file save dialog
   
protected:
//   uchar startup_path[80];
   void CmHelpAbout();       //about function
   void CmLogOn();           //begin loging of the microlan
   void CmLogOff();          //quit saving log to a file
   void CmSystem();          //bring up system dialog
   void CmDiagnostics();     //perform diagnostics
   void CmExit();            //exit function
   void CmComSetup();        //change com_port number
   void SetupWindow();       //setup window function
   void Add_Device();        //function for searching microlan for 1820/1920's
   void CmChange_Temp();     //change temperature thresholds

   DECLARE_RESPONSE_TABLE(TMicroLanClient);
};

DEFINE_RESPONSE_TABLE1(TMicroLanClient, TMDIClient)
   EV_COMMAND(CM_HELPABOUT, CmHelpAbout),   //about funciton
   EV_COMMAND(CM_LOGSETUP, CmSystem),       //system funciton
   EV_COMMAND(CM_LOGON, CmLogOn),           //begin logging
   EV_COMMAND(CM_LOGOFF, CmLogOff),         //stop saving file
   EV_COMMAND(CM_EXIT, CmExit),             //exit the program
   EV_COMMAND(109, CmComSetup),             //comport setup
   EV_COMMAND(CM_DIAGNOSTICS, CmDiagnostics),  //perform diagnostics
   EV_COMMAND(CM_CHANGE_TEMP, CmChange_Temp),  //change temp. function
END_RESPONSE_TABLE;

//==============================================================================
// Setup Window
//==============================================================================
void TMicroLanClient::SetupWindow()
{
   int k;
   TWindow::SetupWindow();
//   getcwd(startup_path, 80);
   names = new name_list;      //create a linked list for the names of the devices
   names->length = 0;

   if(comport == 0)           //check to see if comport supplied on command line
   {
      for(int i = 1; i < 5; i++) //do a smart port search
      {
         handle = TMStartSession(i);  //get a handle
         k = TMSetup(handle);         //setup the one-wire
         if(k == 1)
         {
            comport = i;
            break;
         }  
      }  
   }
   else
   {
      handle = TMStartSession(comport); //get a handle
      k = TMSetup(handle);              //setup the one-wire
   }

   //check to see if need to use previous 
   TDialog* TPrev = new TPreviousDlg(this, PREVIOUS); 
   TPrev->Execute();  
   
   if(previous_flag)
   {                  
      //read in previous session settings
      in_stream = open("exp3.stp", O_RDWR | O_BINARY, S_IREAD | S_IWRITE);

      if(in_stream < 1)  //if can't open file, go ahead and search microlan
         Add_Device();
      else
      {
         read_settings();      //read previous settings from file
         agetree(root);        //age the devices,(used to determine if they were there on last pass)
         set_temp_parameters(root);  //set default temperature parameters
         copy_names(root);     //copy the names to the array, used for the listbox
      }
   }
   else
      Add_Device();   //search microlan for devices
}
//==============================================================================
// Add Devices from the microlan
//==============================================================================
void TMicroLanClient::Add_Device()
{
   int found_one;
   root = NULL;   //clear the root of the binary tree
   rom[0] = 0x00;  //clear the lsb of the rom id


   TDialog* TRen = new TRenameDlg(this, REN_ID); //rename dialog construction
   TDialog* TAdd = new TAddDlg(this, ADD_ANOTHER); //add another construction

   struct tnode far * struct1; //structure used in searching binary tree
   age_flag = 1;         //age flag set to found

   TAdd->Execute();    //execute the add dialog
   if(!TMValidSession(handle))    //make sure its a valid session
   {
      handle = TMStartSession(comport);
      TMSetup(handle);
   }
   if(TMFirst(handle, GB) && add_device)  //search for devices on one wire, add_device is set in TAdd dialog
   {
      TMRom(handle, GB, rom);  //get the rom id
      if(rom[0] == DS1820)    // see if its an 1820
      {
         Ren_ok_flag = FALSE;
         TRen->Execute();       //rename the device
         if(Ren_ok_flag)
         {
            root = addtree(root, rom);    //make it the root, and add it to binary tree
            num_devices++;         // count the number of devices, used in display
            TAdd->Execute();     //ask for another device
         }
         else
            PostQuitMessage(0);  //if cancel exit program
      }
   }
   while(add_device) //while adding devices keep searching if less than 20 devices
   {
      while(TRUE)
      {
         if(!TMValidSession(handle))
         {
            TMEndSession(handle);
            handle = TMStartSession(comport);
            TMSetup(handle);
         }
         found_one = FALSE;
         if(TMFirst(handle, GB))
         {
            rom[0] = 0x00;
            TMRom(handle, GB, rom);
            if(rom[0] == DS1820)
            {
               struct1 = search_tree(root, rom);
               if(struct1 == NULL)
               {
                  Ren_ok_flag = FALSE;
                  TRen->Execute();
                  if(Ren_ok_flag)
                  {
                     found_one = TRUE;
                     addtree(root, rom);
                     num_devices++;
                     TAdd->Execute();
                  }
                  
                  if(num_devices > 20)
                     add_device = FALSE;
                  if(!add_device)
                     break;
               }
            }
         }

         if(!TMValidSession(handle))
         {
            handle = TMStartSession(comport);
            TMSetup(handle);
         }
         while(TMNext(handle, GB))
         {
            rom[0] = 0x00;
            TMRom(handle, GB, rom);
            if(rom[0] == DS1820)
            {
               struct1 = NULL;
               struct1 = search_tree(root, rom);
               if(struct1 == NULL)
               {
                  
                  found_one = TRUE;
                  TRen->Execute();
                  if(Ren_ok_flag)
                  {
                     
                     addtree(root, rom);
                     num_devices++;
                     TAdd->Execute();
                     break;
                  }   
                  if(!add_device || num_devices > 20)
                     break;
               }
            }
         }
         if(!found_one)
         {
            add_device = FALSE;
            break;
         }
         else 
            found_one = FALSE;

      }
      if(!add_device || num_devices > 20)
         break;
   }

   agetree(root);       //age the binary tree
   set_temp_parameters(root);  //set the temperature parameters in the 1820s
   copy_names(root);
   return;             
}

//==============================================================================
// System setup function, calls log setup dialog
//==============================================================================
void TMicroLanClient::CmSystem()
{
   TDlgDialog TD(this, DIALOG_2);
   TD.Execute();
   if(log_file_open_flag == 1)//see if saving to a file
   {
      data = new TFileOpenDialog::TData(               //save file dialog
         OFN_HIDEREADONLY, "LOG files (*.log)|*.log|",
         0,
         0,
         "LOG"
      );
      if ((new TFileOpenDialog(this, *data))->Execute() == IDOK)
      {
         strcpy(filen, data->FileName);    //save the filename
         if(cp != NULL)               //if already logging, change the caption on window
         {
            cp->SetCaption(filen);
            TModule *TM;
            TM = GetModule();
            cp->filename = filen;
            //restart the save file timer
            ID_MYTIMER = ::SetTimer(NULL, 1, 31000, (TIMERPROC)MakeProcInstance((FARPROC)Save, TM->GetInstance()));
         }
      }
   }
   else if(!log_file_open_flag)          //not saving to file
   {
      strcpy(filen, ""); // no log file saving
      if(cp)         //if already logging, change the caption on the window
      {
         cp->SetCaption("");
         if(ID_MYTIMER)   //if we were saving, kill the save timer
         {
            timecount = 0;
            cp->SaveFile();  //save the log file if it needs it
            ::KillTimer(NULL, ID_MYTIMER); // kill the save timer
         }
      }
   }
}
//==============================================================================
// CmDiagnostics-- Perform system diagnostics
//==============================================================================
void TMicroLanClient::CmDiagnostics()
{
   int i;
   if(!TMValidSession(handle)) //check for valid session on Microlan
   {
      handle = TMStartSession(comport); //start a new session
      i = TMSetup(handle);                //perform setup
   }
   else
      i = TMSetup(handle); //perform setup

   //check the return of setup, and post the proper message
   if(i == 1)
      BWCCMessageBox(NULL, "System OK", "Diagnostics", MB_ICONINFORMATION);
   else if(i == 2)
      BWCCMessageBox(NULL, "The MicroLan is shorted or the Com Port adapter is not connected.", "Diagnostics", MB_ICONHAND);
}
//==============================================================================
// About window
//==============================================================================
void TMicroLanClient::CmHelpAbout()
{
   (new TAboutDialog(this, DIALOG_1))->Execute();
}
//==============================================================================
// CmLogOn - Begin logging the network
//==============================================================================
void TMicroLanClient::CmLogOn()
{
   if(!already_loging) //make sure not already logging, only one child per session
   {
      cp = new TDecoChild(*this, filen); //create the child
      cp->Create();
      already_loging = TRUE;  //set the flags
      kill_timer_flag = FALSE;
   }
}
//==============================================================================
// CmLogOff - Stop saving to a file
//==============================================================================
void TMicroLanClient::CmLogOff()
{
   if(cp) 
   {
      if(ID_MYTIMER != NULL)
      {
         log_file_open_flag = 0;  //flag for not saving
         strcpy(filen, "");      //kill the filename variable
         timecount = 0;          //if saving, save the file 
         cp->SaveFile();
         ::KillTimer(NULL, ID_MYTIMER); //kill the save timer
         ID_MYTIMER = FALSE;     //clear the timer id
      }
      cp->SetCaption("");
   }
}
//==============================================================================
// Exit function
//==============================================================================
void TMicroLanClient::CmExit()
{
   //save the previous session's settings
   out_stream = open("exp3.stp", O_CREAT | O_TRUNC | O_RDWR | O_BINARY, S_IREAD|S_IWRITE);
   if(out_stream > 0)
      save_setup(root);
   close(out_stream);
   //kill any timers not already killed
   if(timer_set_flag)
      cp->KillTimer(NULL);
   if(ID_MYTIMER != NULL)
   {
      ::KillTimer(NULL, ID_MYTIMER);
      timecount = 0;
      cp->SaveFile();
   }
   PostQuitMessage(0);  //post quit message
}
//==============================================================================
// CmComSetup - Change comport number
//==============================================================================
void TMicroLanClient::CmComSetup()
{
   TComSetupDlg TCom(this, DIALOG_4);
   TCom.Execute();
}
//==============================================================================
// CmChang_Temp - Change the temperature windows of the 1820/1920's
//==============================================================================
void TMicroLanClient::CmChange_Temp()
{
   TChangeTempDlg TChange(this, CHANGE_TEMP);
   TChange.Execute();
}
// ===========================================================
// The MDI application class
// ===========================================================
class TMicroLanApp: public TApplication {
public:
  TMicroLanApp(const char far* name)  //constructor
    : TApplication(name) {};
  void InitMainWindow();   //sets up the main window
  bool CanClose();
private:
  TMicroLanClient* client;    //pointer to client window class
  TControlBar* toolbar;       //pointer to control bar
  TStatusBar* statusLine;     //status bar
};
//==============================================================================
// InitMainWindow - sets up toolbar and control bar and launches client class
//==============================================================================
void TMicroLanApp::InitMainWindow()
{
   // Construct the MDI client window
   client = new TMicroLanClient;

   // Construct MDI frame, passing client window object
   
   TDecoratedMDIFrame * frame = new TDecoratedMDIFrame(
    "Temperature Monitor with DS1820",  // Caption
    MENU_1,      // Required menu resource identifier
    *client,      // Required client window object
    TRUE);        // Optional track menu selections


   // Construct a toolbar with icons for various commands
   toolbar = new TControlBar(frame);
   toolbar->Insert(
      *new TButtonGadget(LOGON, CM_LOGON));
   toolbar->Insert(
      *new TButtonGadget(LOGOFF, CM_LOGOFF));
   toolbar->Insert(
      *new TSeparatorGadget(16));
   toolbar->Insert(
      *new TButtonGadget(CM_SETUP, CM_LOGSETUP));
   toolbar->Insert(
      *new TButtonGadget(CHANGE_TEMP, CM_CHANGETEMP));
   toolbar->Insert(
      *new TButtonGadget(IDB_DIAGNOSTICS, CM_DIAGNOSTICS));
   toolbar->Insert(
      *new TSeparatorGadget(16));
   toolbar->Insert(
      *new TButtonGadget(IDB_INFO, CM_HELPABOUT));
   toolbar->Insert(
      *new TButtonGadget(IDB_FILEEXIT, CM_EXIT));
   toolbar->Insert(
      *new TSeparatorGadget(8));

   // Enable status line messages when mouse touches a button
   toolbar->SetHintMode(TGadgetWindow::EnterHints);

   
   // Insert the toolbar into the frame window
   frame->Insert(*toolbar, TDecoratedFrame::Top);

   // Create status line with function keys legend
   statusLine = new TStatusBar(frame, TGadget::Recessed,
    TStatusBar::CapsLock | TStatusBar::NumLock |
    TStatusBar::ScrollLock | TStatusBar::Overtype);
   frame->Insert(*statusLine, TDecoratedFrame::Bottom);

   // Assign the decorated MDI frame as the program's main window
   SetMainWindow(frame);
   MainWindow->SetIcon(this, A_N_T_ICON);
   strcpy(filen, "");
}

bool TMicroLanApp::CanClose()
{
   if(timer_set_flag)
      cp->KillTimer(NULL);
   if(ID_MYTIMER != NULL)
   {
      ::KillTimer(NULL, ID_MYTIMER);
      timecount = 0;
      cp->SaveFile();
   }
   return TRUE;
}

#pragma argsused

// Main program
int OwlMain(int argc, char* argv[])
{
   if(argc > 1) //check for command line parameters
   {
      comport = atoi(argv[1]);    //get comport if provided
      if(comport < 0 || comport > 16)
         comport = 0;   
   }
   else
      comport = 0;
   //application construction
   TMicroLanApp app("Temperature Monitor with DS1820");
   return app.Run();
}

