/******************************************************************************\
*  Copyright (C) 1995 Dallas Semiconductor Corporation.                        *
*  All rights Reserved. Printed in U.S.A.                                      *
*  This software is protected by copyright laws of                             *
*  the United States and of foreign countries.                                 *
*  This material may also be protected by patent laws of the United States     *
*  and of foreign countries.                                                   *
*  This software is furnished under a license agreement and/or a               *
*  nondisclosure agreement and may only be used or copied in accordance        *
*  with the terms of those agreements.                                         *
*  The mere transfer of this software does not imply any licenses              *
*  of trade secrets, proprietary technology, copyrights, patents,              *
*  trademarks, maskwork rights, or any other form of intellectual              *
*  property whatsoever. Dallas Semiconductor retains all ownership rights.     *
\******************************************************************************/

//==============================================================================
// mstrcpy - string copy that works with any memory model
//==============================================================================
void mstrcpy(char far *dest, char far *src)
{
   int i = 0;

   do
   {
      dest[i] = src[i];
   }
   while (src[i++] != 0);
}
//==============================================================================
// memcpy - any memory model
//==============================================================================
void mmemcpy(uchar far *dest, uchar far *src, uchar transnum)
{
   uchar jll;

  if (src == NULL)
    dest = NULL;
  else
    for (jll = 0; jll < transnum; jll++)
      *(dest + jll) = *(src + jll);
}

//==============================================================================
// memcmp - any memory model
//==============================================================================
int mmemcmp(uchar far *m1, uchar far *m2, int n)
{
  int i;

  for(i=0;i<n;i++)
  {
    if(*(m1 + i) == *(m2 + i))
      continue;
      
    if(*(m1 + i) < *(m2 + i))
      return -(++i); // return negative of index+1 at which failure occured

    return ++i; // return index+1 at which failure occured
  }
  return 0;
}

//==============================================================================
// addtree - add a structure to binary tree - recursive
//==============================================================================
struct tnode *addtree(struct tnode far *p, int rom1[8])
{
   int cond;

   if (p == NULL)
   {
      if((p = talloc()) != NULL)
      {
         mmemcpy((uchar far *) p->ROMID, (uchar far *) rom1, 8*sizeof(int));
         //setup values for device
         p->maxtemp = upper_alarm_temp; //alarm temperatures
         p->mintemp = lower_alarm_temp;
         mstrcpy(p->word, new_name);    //copy device name
         p->ageflag = age_flag;         //age flag - device found/not found
         p->count = num_devices;        //number of devices
         p->set_alarm = alarm_on;       //alarm_on or not
         p->left = p->right = NULL;     //NULL for left/right pointer
         p->present[0] = 0;             //string for found
         p->temperature[0] = 0;         //temperature string
      }
      
   }
   else if((cond = mmemcmp((uchar far *) rom1, (uchar far *) p + 25, 8*sizeof(int))) == 0)
      p->count++; //increment device count
   else if (cond < 0)
      p->left = addtree(p->left, rom1); //add to the left node
   else
      p->right = addtree(p->right, rom1); //add to the right node
   return p;
}

//==============================================================================
// agetree - device found or nor - recursive
//==============================================================================
int agetree(struct tnode *p)
{
   if (p != NULL)
   {
      agetree(p->left);
      //for this exp., no choice on switchflag.  Left over form exp1&2
      if(!p->ageflag && !switchflag)
      {
         strcat(szmissing, p->word);
         strcpy(p->present, "MISSING");
         strcat(szmissing, " ");
         p->temperature[0] = 0;
         //return 0;
      }
      else if(p->ageflag && switchflag)
      {
         strcat(szmissing, p->word);
         strcpy(p->present, "MISSING");
         strcat(szmissing, " ");
         p->temperature[0] = 0;
      }
      else
         strcpy(p->present, p->temperature); //copy devices temperature
      if(!switchflag)
         p->ageflag = 0;    //default ageflag
      else if(switchflag)
         p->ageflag = 1;
      agetree(p->right);
   }
   if(szmissing[0] != 0) //return 0 if any devices not found
      return 0;   
   return 1;   
}

//==============================================================================
// set_temp_parameters - recursive - set temp for each 1820
//==============================================================================
int set_temp_parameters(struct tnode *p)
{
   changing_temp = TRUE;
   if(!TMValidSession(handle))      //check session
   {
      handle = TMStartSession(comport);
      TMSetup(handle);
   }
   if (p != NULL)
   {
      set_temp_parameters(p->left); //set left node

      if(p->set_alarm)     //if alarm is on, not a given choice.
      {
         TMTouchReset(handle);       //reset one wire
         TMRom(handle, GB, p->ROMID); //access this device
         TMAccess(handle, GB);
         TMTouchByte(handle, 0x4E);   //write scratchpad
         TMTouchByte(handle, p->maxtemp); //pass upper temp.
         TMTouchByte(handle, p->mintemp); //pass lower temp.
         TMTouchReset(handle);        //reset one wire
         TMAccess(handle, GB);        //access device
         TMTouchByte(handle, 0x48);   //copy scratchpad
         wait_counts(1200L);          //hold one wire high for at least 10ms
      }

      set_temp_parameters(p->right); //set right node temp.
   }
   changing_temp = FALSE; //through changing temperatures
   return 1;
}

//==============================================================================
// agetreenull - set ageflags to zero - recursive
//==============================================================================
int agetreenull(struct tnode *p)
{
   if (p != NULL)
   {
      agetreenull(p->left);
      p->ageflag = 0;
      agetreenull(p->right);
   }
   return TRUE;
}

//==============================================================================
// search_tree for the given romid
//==============================================================================
struct tnode *search_tree(struct tnode *p, int rom1[8])
{
   if(!p) return p;
   int cond;
   //compare the romid with the current node
   while ((cond = mmemcmp((uchar *) p->ROMID, (uchar *) rom1, 16)) != 0)
   {
      if(cond > 0) p = p->left; //if > check left node
      else p = p->right;      //check right node
      if(p == NULL)
         break;
   }
   if(cond == 0 && p != NULL && !switchflag)
      p->ageflag = 1;            //device found
   else if(cond == 0 && p != NULL && switchflag)
      p->ageflag = 1;           //device found
   return p;
}

//==============================================================================
// search the tree by the devices alias
//==============================================================================
struct tnode *search_tree_string(struct tnode *p, char *pstring)
{
   if(!p) return p; //no tree return

   if(found_string != NULL) //device already found
      return found_string;

   if (p != NULL)
   {
      search_tree_string(p->left, pstring); //step through each node, and compare
      if((strcmp(p->word, pstring)) == 0)
      {
         found_string = p;        //device found
         return found_string;
      }
      if(found_string != NULL)
         return found_string;   //device found                  
      search_tree_string(p->right, pstring);  //go to right node
   }
   return found_string;
   
}

//==============================================================================
// Initialize the node in memory
//==============================================================================
struct tnode far *talloc()
{
   return new struct tnode;
}

//==============================================================================
// Check_Tree - check binary tree to see if devices are found
//==============================================================================
int Check_Tree()
{
   rom[0] = 0x00;   //clear lsb
   if(!TMValidSession(handle)) //check session
   {
      handle = TMStartSession(comport);
      TMSetup(handle);
   }
   if(!switchflag)      //search tree based on switchflag
   {
      if(TMFirst(handle, GB))  //search for first device
      {
         TMRom(handle, GB, rom); //get devices rom_id
         search_tree(root, rom);
      }
      while(TMNext(handle, GB))    //while devices still found
      {
         rom[0] = 0x00;            //clear lsb of rom_id
         TMRom(handle, GB, rom);   //get rom_id of device
         search_tree(root, rom);   //mark tree that device found
      }   
   
      return(agetree(root));       //return agetree
   }
   else
   {
      agetreenull(root);   //if switchflag, set ageflags to 0
      if(TMFirst(handle, GB))
      {
         TMRom(handle, GB, rom);
         search_tree(root, rom);
      }
      else
         return (agetree(root));
      while(TMNext(handle, GB))
      {
         rom[0] = 0x00;
         TMRom(handle, GB, rom);
         search_tree(root, rom);
      }   
   
      return(agetree(root));
   }   
}

//==============================================================================
// alarm_search - search onewire for alarming devices
//==============================================================================
int alarm_search()
{
   int alarm_rom[8], i, found= FALSE;
   struct tnode *struct1;

   if(!TMValidSession(handle)) //get for valid session
   {
      handle = TMStartSession(comport);
      TMSetup(handle);
   }
   i = TMFirstAlarm(handle, GB);    //do a first alarm

   if(i != 0)  //if found
   {
      alarm_rom[0] = 0;             //clear alarm_rom lsb
      TMRom(handle, GB, alarm_rom); //get the devices romid
      struct1 = search_tree(root, alarm_rom);   //search the tree for the device
      if(struct1 != NULL) //if its found
      {
         alarm_flag = TRUE;         //set the global alarm flag
         struct1->alarming = TRUE;  //this device is alarming
         strcpy(alarming_devices, struct1->word);   //copy device name to alarming string
         strcat(alarming_devices, " ");  //add a space after name
         strcat(alarming_devices, struct1->temperature); //add the devices temp.
         found = TRUE;
      }
      else
         found = FALSE;
      while(TMNextAlarm(handle, GB)) //do this until no more alarming devices
      {
         alarm_rom[0] = 0;
         TMRom(handle, GB, alarm_rom);
         struct1 = search_tree(root, alarm_rom);
         if(struct1 != NULL)
         {
            struct1->alarming = TRUE;
            strcat(alarming_devices, " ");
            strcat(alarming_devices, struct1->word);
            strcat(alarming_devices, " ");
            strcat(alarming_devices, struct1->temperature);
         }
      }
   }
  return found;   
} 

//==============================================================================
// copy_names - copy names to linked list of names
//              used for the listbox
//==============================================================================
void copy_names(struct tnode *p)
{
   if (p != NULL)
   {
      copy_names(p->left);

         mstrcpy(names->name[names->length], p->word);
         names->length++;
         
      copy_names(p->right);
   }
   return;      
}

//==============================================================================
// save_setup - save the session info
//==============================================================================
void save_setup(struct tnode *p)
{
   write(out_stream, (void *)&comport, sizeof(comport));
   write(out_stream, (void *)&flag_celsius, sizeof(flag_celsius));
   write(out_stream, (void *)&switchflag, sizeof(switchflag));
   write(out_stream, (void *)&log_pass_flag, sizeof(log_pass_flag));
   write(out_stream, (void *)&global_time_interval, sizeof(global_time_interval));
   write(out_stream, (void *)filen, 80);
   write(out_stream, (void *)&log_file_open_flag, sizeof(log_file_open_flag));
   save_setup1(p);
   write(out_stream, (void *)&num_devices, sizeof(num_devices));
}

//==============================================================================
// save_setup1 - save the binary tree - recursive
//==============================================================================
void save_setup1(struct tnode *p)
{
   if (p!=NULL)
   {
      save_setup1(p->left);     //save left node
         write(out_stream, (void *) p, sizeof(tnode)); //write it to a file
      save_setup1(p->right);    //save right node
   }
   return;
}

//==============================================================================
// read previous settings
//==============================================================================
void read_settings(void)
{
   read(in_stream, (void *)&comport, sizeof(comport));
   read(in_stream, (void *)&flag_celsius, sizeof(flag_celsius));
   read(in_stream, (void *)&switchflag, sizeof(switchflag));
   read(in_stream, (void *)&log_pass_flag, sizeof(log_pass_flag));
   read(in_stream, (void *)&global_time_interval, sizeof(global_time_interval));
   read(in_stream, (void *)filen, 80);
   read(in_stream, (void *)&log_file_open_flag, sizeof(log_file_open_flag));

   read_settings1();
   read(in_stream, (void *)&num_devices, sizeof(num_devices));
}

//==============================================================================
// read previous binary tree
//==============================================================================
void read_settings1(void)
{
   struct tnode *p;
   p = new tnode;
   if(in_stream == NULL)  //make sure we have a file pointer
      return;
   while((read(in_stream, (void *) p, sizeof(tnode))) == sizeof(tnode))
   {
      if(root == NULL)
      {
         upper_alarm_temp = p->maxtemp;
         lower_alarm_temp = p->mintemp;
         mstrcpy(new_name, p->word);
         age_flag = p->ageflag;
         alarm_on = p->set_alarm;
         root = addtree(root, p->ROMID);
         num_devices++;
      }
      else
      {
         upper_alarm_temp = p->maxtemp;
         lower_alarm_temp = p->mintemp;
         mstrcpy(new_name, p->word);
         age_flag = p->ageflag;
         alarm_on = p->set_alarm;
         addtree(root, p->ROMID);
         num_devices++;
      }
   }
   close(in_stream);
   return;
}

//==============================================================================
// fill the list box array with passes info - recursive
//==============================================================================
int fill_array_with_info(struct tnode *p)
{
   if(p!= NULL)
   {
      fill_array_with_info(p->left);  //do left node
      strcpy(device_array[p->count], p->word);  //copy the device name
      strcpy(device_array[p->count+20], p->present);  //device found/not found
      if(!strcmp(p->present, "MISSING")) //not found
         strcpy(device_array[p->count+40], ""); //don't print a temperature
      else if(p->alarming)
         strcpy(device_array[p->count+40], "ALARMING"); //device alarming
      else 
         strcpy(device_array[p->count+40], "Temperature OK"); //temperature ok
      p->alarming = FALSE;  //reset the alarm flag
      fill_array_with_info(p->right); //do the right node
   }
   return TRUE;  
}

//==============================================================================
// wait_counts - uses the 8250 counter to wait an exact amount of time
//==============================================================================
static void wait_counts(ulong wait_count)
{
   unsigned short icount, ccount = 0, lcount;
   outp(0x43, 0);                              // Freeze the initial count 
   lcount = inp(0x40);                           // Read low byte of count 
   icount = (inp(0x40) << 8) | lcount;           // Read the initial count 
   do
   {
      outp(0x43, 0);                           // Freeze the current count 
      lcount = inp(0x40);                        // Read low byte of count 
      ccount = (inp(0x40) << 8) | lcount;        // Read the initial count 
   }                                                                       
   while ((unsigned short) (icount - ccount) < wait_count);          // Burn cycles
   
}

//==============================================================================
// convert the float to a string
//==============================================================================
char *ftoa(double d, char *buff, int numdecplaces)
{
  int i;
  long ltemp;
  char tempbuff[80];
  double mult;


  buff[0] = '\0';
  mult = 100.0;
  ltemp = (long)(mult*d);
// get integer part
  strcpy(buff,ltoa(ltemp/ (long) mult,tempbuff,10));
  strcat(buff,".");
// get decimal part
  ltemp = labs(ltemp);
  ltemp %= (long)mult;

  if(numdecplaces == 1)
  {
   ltemp += 5;
   ltemp /= 10;
  }

  for (i=0;i<(numdecplaces-1);i++)
  {
    if (ltemp < pow10(i+1))
      strcat (buff,"0");
  }
  strcat(buff,ltoa(ltemp,tempbuff,10));
  return buff;
}
