/******************************************************************************\
*  Exp4                                                                        *
*  Copyright (C) 1995 Dallas Semiconductor Corporation.                        *
*  All rights Reserved. Printed in U.S.A.                                      *
*  This software is protected by copyright laws of                             *
*  the United States and of foreign countries.                                 *
*  This material may also be protected by patent laws of the United States     *
*  and of foreign countries.                                                   *
*  This software is furnished under a license agreement and/or a               *
*  nondisclosure agreement and may only be used or copied in accordance        *
*  with the terms of those agreements.                                         *
*  The mere transfer of this software does not imply any licenses              *
*  of trade secrets, proprietary technology, copyrights, patents,              *
*  trademarks, maskwork rights, or any other form of intellectual              *
*  property whatsoever. Dallas Semiconductor retains all ownership rights.     *
\******************************************************************************/

//==============================================================================
// Child Class:  Only one child window is allowed at a time.  SDI would have 
// worked.
//==============================================================================
class TDecoChild: public TMDIChild {
public:
   time_t timer;
   struct tm *tblock;   
   TDecoChild(TMDIClient& parent, const char far *name);
   ~TDecoChild();
   void SetupWindow();
   int Unnamed() { return filename.length() == 0; }
   int SaveFile();
   void EvTimer(UINT timerId);
   void EvSize(UINT sizeType, TSize& size);
   void Paint(TDC& dc, BOOL, TRect&);
   int numRows, numCols;
   string filename;     //used to store the logfile name;
   double temp10;
   TIArrayAsVector<string> sArray; //array for devices.
   int first_pass;
private:
   int cyChar;
   long width, height;
   int rowSep, colSep;
   DECLARE_RESPONSE_TABLE(TDecoChild);
};

DEFINE_RESPONSE_TABLE1(TDecoChild, TMDIChild)
   EV_WM_TIMER,
   
END_RESPONSE_TABLE;

//============================================================================
// Constructor
//============================================================================
TDecoChild::TDecoChild(TMDIClient& parent, const char far *name)
  : TMDIChild(parent, name), sArray(SARRAY_HIGH, SARRAY_LOW, SARRAY_DELTA)
{
   filename = name;                    //copy filename
   Attr.Style |= MDIS_ALLCHILDSTYLES;  //allow all window styles
   Attr.Style &= ~WS_MAXIMIZEBOX;      //no maximize/minimize box
   Attr.Style &= ~WS_MINIMIZEBOX;
   Attr.Style |=WS_MAXIMIZE;           //maximize window
   numRows = numCols = 0;
   rowSep = colSep = 0;
   first_pass = TRUE;
   numRows = 0;
      
   TModule *TM;                        //needed for timer
   TM = GetModule();

   // timer for saving file every few minutes.  time controlled by a define statement;
   if(strcmp(name, "") != 0)
      ID_MYTIMER = ::SetTimer(NULL, 1, 30000, (TIMERPROC)MakeProcInstance((FARPROC)Save, TM->GetInstance())); 
}

//============================================================================
// Destructor
//============================================================================
TDecoChild::~TDecoChild()
{
      //kill all timers and switch off 2405s
      if(timer_set_flag)
      {
         KillTimer(ID_EVTIMER);
         timer_set_flag = FALSE;
         ID_EVTIMER = 0;
      }
      if(ID_TEMPTIMER)
      {
         ::KillTimer(NULL, ID_TEMPTIMER);
         ID_TEMPTIMER = 0;
      }
      if(ID_MYTIMER != NULL)
      {
         ::KillTimer(NULL, ID_MYTIMER);
         timecount = 0;
         cp->SaveFile();
         ID_MYTIMER = 0;
      }
      SwitchOff(upper_cntl);
      SwitchOff(lower_cntl);
      return;
}

//============================================================================
//  Windows timer(not a class member) used to transfer to class save funciton
//============================================================================
WORD FAR PASCAL Save(HWND, WORD, WORD, LONG)
{
   cp->SaveFile();
   return 0;
}

//============================================================================
// Save the log file
//============================================================================
int TDecoChild::SaveFile()
{
   char c[80];
   const char* cst;
   if(timecount == 0)
   {
      //check to make sure file needs to be saved.
      if(currentRows != numRows)     
      {
         FILE *fp;
         fp = fopen(filename.c_str(), "a+");
         //save only the new lines since last save
         for(int i = currentRows; i < numRows; i++)   
         {   
            cst = sArray[i]->c_str();
            strcpy(c, cst);
            int len;
            c[(len = strlen(c))] = 0x0a;
            c[len+1] = 0;
            fwrite(c, strlen(c), 1, fp);
         }
         currentRows = numRows;
         timecount = WAIT_TIME;  //reset countdown
         fclose(fp);
      }
      else
         timecount = WAIT_TIME;  //doesn't need to be saved
   }
   else                    //not time to save yet
      --timecount;
   return TRUE;
}

//============================================================================
// EvTimer - polls the system for devices and prints status to sArray
//============================================================================
void TDecoChild::EvTimer(UINT)
{
   char s[640], temp_string[500];
   int j;

   temp_string[0] = 0;
   
   if(!finished_flag)  //make sure not taking temperature
      return;
   if(kill_timer_flag)
   {
      KillTimer(NULL);
      ::KillTimer(NULL, ID_MYTIMER);
      return;
   }

   if(num_waits == 0)  //check timer interval for polling microlan
   {
      szmissing[0] = 0;           //string for missing devices
      if(Check_Tree())            //check for missing devices
      {
         TModule *TM;             //for timer
         TM = GetModule();
         if(!TMValidSession(handle))   //check handle
         {
            handle = TMStartSession(comport);
            TMSetup(handle);
         }
         TMTouchReset(handle);     //send reset to 1820's
         j = TMFirst(handle, GB);  //find first device on onewire
         if(j == 1)
         {
            rom[0] = 0x00;         //clear the romid
            TMRom(handle, GB, rom);    //access the byte
            TMTouchByte(handle, 0x55); //send skip rom command
            TMRom(handle, GB, rom);
            TMAccess(handle, GB);
            TMTouchByte(handle, 0x44); //send convert T
            finished_flag = FALSE;     //not finished taking temp

            //turn on 1 second timer for holding microlan high
            ID_TEMPTIMER = ::SetTimer(NULL, 2, 1000, (TIMERPROC)MakeProcInstance((FARPROC)Temperature, TM->GetInstance()));
         }

         //determine which logging procedure is in use.
         if((!log_pass_flag && !alarm_flag) || first_pass)
         {
            first_pass = FALSE;
            numRows++;
            strcpy(s, sz);
            timer = time(NULL);
            tblock = localtime(&timer);
            strcat(s, asctime(tblock));
            s[strlen(s) - 1] = 0;
            sArray.Add(new string(s));
            last_pass = FALSE;
         }
         else if((log_pass_flag && alarm_flag) || !log_pass_flag)
         {
            numRows++;
            strcpy(s, alarming_devices);
            strcat(s, " ");   
            timer = time(NULL);
            tblock = localtime(&timer);
            strcat(s, asctime(tblock));
            s[strlen(s) - 1] = 0;
            strcat(s, " ");
              
            //1820 out of range, append Cooler or Heater to its temperature
            if(flag_celsius)
            {
               if(temp10 > ((double)upper_alarm_temp*9.0/5.0 + 32.0) )
               {
                  strcat(s, " Cooler");
               }
              else if(temp10 < ((double)lower_alarm_temp*9.0/5.0 + 32.0))
              {
                  strcat(s, " Heater");
              }
            }        
            else
            {
               if(temp10 > (double)upper_alarm_temp)
               {
                  strcat(s, " Cooler");
               }
               else if(temp10 < (double)lower_alarm_temp)
               {
                  strcat(s, " Heater");
               }
            }
            sArray.Add(new string(s));
            alarm_flag = FALSE;        //clear alarm_flag for next pass
         } 
         strcpy(sz, "");
      }
      else  //devices missing
      {
         //determing which logging procedure is in use;
         if((log_pass_flag && last_pass == FALSE) || log_pass_flag == 0)
         {
            numRows++;
            strcpy(s, "Device(s) Not Found: ");
            strcat(s, szmissing);
            timer = time(NULL);
            tblock = localtime(&timer);
            strcat(s, asctime(tblock));
            s[strlen(s) - 1] = 0;
            sArray.Add(new string(s));
            last_pass = TRUE;
         }
         
      }
      Invalidate(TRUE);
      num_waits = old_num_waits;
  }
  else
   num_waits--;
}


//============================================================================
// SetupWindow - intialize all variables
//============================================================================
void TDecoChild::SetupWindow()
{
   TFrameWindow::SetupWindow();
   TEXTMETRIC tm;
   
   dc = new TClientDC(*this);
   dc->GetTextMetrics(tm);
   cyChar = tm.tmHeight +tm.tmExternalLeading; //determine line spacing
   
   delete dc;

//release the timer function
   
   ID_EVTIMER = SetTimer(1, global_time_interval*1000);
   timer_set_flag = TRUE;
   return;
}


//============================================================================
//  EvSize - no purpose, could be added for scrollbars
//============================================================================
void TDecoChild::EvSize(UINT sizeType, TSize& size)
{
   if(!IsIconic())
   {
      Invalidate(FALSE);
   }
   TFrameWindow::EvSize(sizeType, size);
}

//============================================================================
// Paint - function to paint on the screen
//============================================================================
void TDecoChild::Paint(TDC& dc, BOOL, TRect&)
{
   TRect r;
   int row, x, y;
   const char* c;
   int linenum = 0;
   if(numRows == 0) return;

   dc.GetClipBox(r);

   if(numRows < 20)     //only printing 20 rows on screen at a time
      row = 0;
   else row = numRows - 20;
   for (row; row < numRows; row++)
   {
      x = colSep;
      y = linenum * cyChar;
      linenum++;
      c = sArray[row]->c_str();
      dc.TextOut(x, y, c, strlen(c));
   }
}


//ev_timer function used to measure temperature of all devices on the one-wire;
//while this function is in use, the TDecoChild::EvTimer function cannot take place;
//This function should be modified to behave like the same
//function in exp5.  It sends the convert T to all devices, this is only
//because it was copied and left unmodified from exp3.
WORD FAR PASCAL Temperature(HWND, WORD, WORD, LONG)
{
   char buf[10], buffer[60];
   int scratchpad[8], next_device = FALSE;
   tnode *struct1;
   int rom2[8];
   if(!changing_temp)
   {
   if(!TMValidSession(handle))
      {
         TMEndSession(handle);
         handle = TMStartSession(comport);
         TMSetup(handle);
      }

      memcpy(rom2, rom, sizeof(rom2));
      TMTouchByte(handle, 0x55);
      TMRom(handle, GB, rom);
      TMAccess(handle, GB);
      TMTouchByte(handle, 0xBE);
      for(int i = 0; i < 9; i++)
         scratchpad[i] = TMTouchByte(handle, 0xFF);
      
      cp->temp10 = (double)(scratchpad[0] >> 1);
      if(scratchpad[0] & 0x01)
         cp->temp10 = cp->temp10 + 0.5;

      if(flag_celsius)
         cp->temp10 = cp->temp10 * 9.0 / 5.0 + 32.0;

      ftoa(cp->temp10, buf, 2); 
      if(!flag_celsius)
         strcat(buf, "C");
      else
         strcat(buf, "F");
      struct1 = search_tree(root, rom);                  

      if(scratchpad[1] == 0xFF && struct1 != NULL)
      {
         mstrcpy(struct1->temperature, "-");
         strcat(struct1->temperature, buf);
      }
      else if(struct1 != NULL)
         mstrcpy(struct1->temperature, buf);

      strcpy(buffer, struct1->word);
      strcat(buffer, " ");
      strcat(buffer, buf);
      strcat(sz, buffer);
      strcat(sz, " ");

      while(TMNext(handle, GB)==1)
      {
         rom[0] = 0x00;
         TMRom(handle, GB, rom);
         if(memcmp(rom2, rom, sizeof(rom)) != 0)
         {
            struct1 = search_tree(root, rom);
            if(struct1 != NULL)
            {
               next_device = TRUE;
               TMTouchByte(handle, 0x55);
               TMRom(handle, GB, rom);
               TMAccess(handle, GB);
               TMTouchByte(handle, 0x44);
               break;
            }
            next_device = FALSE;
         }
      }
      if(!next_device)
      {
         finished_flag = TRUE;
         first_time = FALSE;
         ::KillTimer(NULL, ID_TEMPTIMER);
         if(alarm_on)
            if(alarm_search())
               sz[0] = 0;
         if(flag_celsius)
         {
            if(cp->temp10 > ((float)upper_alarm_temp*9.0/5.0 + 32.0) )
            {
               SwitchOn(upper_cntl);
               SwitchOff(lower_cntl);
            }
            else if(cp->temp10 < ((float)lower_alarm_temp*9/5 + 32))
            {
               SwitchOn(lower_cntl);
               SwitchOff(upper_cntl);
            }
            else
            {
               SwitchOff(lower_cntl);
               SwitchOff(upper_cntl);
            }
         }        
         else 
         {
            if(cp->temp10 > upper_alarm_temp)
            {
               SwitchOn(upper_cntl);
               SwitchOff(lower_cntl);
            }
            else if(cp->temp10 < lower_alarm_temp)
            {
               SwitchOn(lower_cntl);
               SwitchOff(upper_cntl);
            }
            else
            {
               SwitchOff(lower_cntl);
               SwitchOff(upper_cntl);
            }
         }
      }
   }
   return 0;
}


