/******************************************************************************\
*  Exp4                                                                        *
*  Copyright (C) 1995 Dallas Semiconductor Corporation.                        *
*  All rights Reserved. Printed in U.S.A.                                      *
*  This software is protected by copyright laws of                             *
*  the United States and of foreign countries.                                 *
*  This material may also be protected by patent laws of the United States     *
*  and of foreign countries.                                                   *
*  This software is furnished under a license agreement and/or a               *
*  nondisclosure agreement and may only be used or copied in accordance        *
*  with the terms of those agreements.                                         *
*  The mere transfer of this software does not imply any licenses              *
*  of trade secrets, proprietary technology, copyrights, patents,              *
*  trademarks, maskwork rights, or any other form of intellectual              *
*  property whatsoever. Dallas Semiconductor retains all ownership rights.     *
\******************************************************************************/

//--------------------------Com Setup Dialog------------------------------------

class TComSetupDlg : public TDialog
{
   public:
      TComSetupDlg(TWindow* parent, int);
      void SetupWindow();
      void CloseWindow(int);
   DECLARE_RESPONSE_TABLE(TComSetupDlg);
};

DEFINE_RESPONSE_TABLE1(TComSetupDlg, TDialog)
END_RESPONSE_TABLE;

//============================================================================
// Constructor
//============================================================================
TComSetupDlg::TComSetupDlg(TWindow* parent, int DlgResNum)
   : TDialog(parent, DlgResNum), TWindow(parent)
{

}
//============================================================================
//SetupWindow
//============================================================================
void TComSetupDlg::SetupWindow()
{
   
   if(comport == 1)
      CheckRadioButton(101, 104, IDC_COM1);
   else if(comport == 2)
      CheckRadioButton(101, 104, IDC_COM2);
   else if(comport == 3)
      CheckRadioButton(101, 104, IDC_COM3);
   else
      CheckRadioButton(101, 104, IDC_COM4);
}
//============================================================================
//CloseWindow
//============================================================================
void TComSetupDlg::CloseWindow(int)
{
   if(IsDlgButtonChecked(101) == TRUE)
      comport = 1;
   else if(IsDlgButtonChecked(102) == TRUE)
      comport = 2;
   else if(IsDlgButtonChecked(103) == TRUE)
      comport = 3;
   else if(IsDlgButtonChecked(104) == TRUE)
      comport = 4;
   TWindow::CloseWindow();
   handle = TMStartSession(comport);
   
}


//-------------------------FileOpen Dialog Class for log file-------------------
class TComDlgWindow : public TFrameWindow
{
   public:
      TComDlgWindow(TWindow* parent, const char* title);

   DECLARE_RESPONSE_TABLE(TComDlgWindow);

};

DEFINE_RESPONSE_TABLE1(TComDlgWindow, TFrameWindow)

END_RESPONSE_TABLE;

//============================================================================
//Constructor
//============================================================================
TComDlgWindow::TComDlgWindow(TWindow* parent, const char* title)
   : TFrameWindow(parent, title), TWindow(parent, title)
{
   AssignMenu(1);
}


//-----------------------------TDialog Class for log setup----------------------
class TDlgDialog : public TDialog
{

   public:
      float time_interval;
      TDlgDialog(TWindow* parent, int);
      DECLARE_RESPONSE_TABLE(TDlgDialog);
      void SetupWindow();
      void CloseWindow(int retValue = IDCANCEL);
      void EvVScroll(UINT code, UINT pos, HWND wnd);
};
DEFINE_RESPONSE_TABLE1(TDlgDialog, TDialog)
      EV_WM_VSCROLL,
END_RESPONSE_TABLE;

//============================================================================
//Constructor
//============================================================================
TDlgDialog::TDlgDialog(TWindow* parent, int DlgResNum)
   :  TWindow(parent), TDialog(parent, DlgResNum)
{
   log_file_open_flag = 0;
   time_interval = (float)global_time_interval/ 100;
}
//============================================================================
//EvVScroll - handles the changeing of the time interval
//============================================================================
void TDlgDialog::EvVScroll(UINT code, UINT, HWND)
{
   char buff[40];

   DestroyCaret();

   switch(code)
   {
      case 0:
         if(time_interval < MAXTIMEINTERVAL)
         {
            if(time_interval == 0.60)
            
               time_interval = 1.00;
            
            if(time_interval < .60)
            {

               time_interval = time_interval + 0.05;

              if((time_interval - 0.60) > 0.00 && (time_interval - 0.60) < 0.04)
                  time_interval = 1.00;
            }
            else
               time_interval = time_interval + 1.00;
         }
      break;
      case 1:
         if(time_interval > MINTIMEINTERVAL)
         {

            if((time_interval - 1.00) < 0.04 && (time_interval - 1.00) > - 0.04)
            
               time_interval = .60;
            
            if(time_interval < .65)

               time_interval -= 0.0500;

            else
               time_interval--;
         }
      break;
      default:
         break;
      }
      if(time_interval == 0.6)
         time_interval = 1.00;
     
      if(time_interval < .06)
         strcpy(buff, "0.00");
      else
         gcvt(time_interval, 4, buff);
      for(int i = 0; i < 10; i++)
         if(buff[i] == '.')
            buff[i] = ':';
      if(time_interval > .61)
         strcat(buff, ":00");
      else if(time_interval > .01)
      {
         if(strlen(buff) < 4)
            strcat(buff, "0");
      }

      SetDlgItemText(DISPLAY_TIME_INTERVAL, buff);

}
//============================================================================
// SetupWindow
//============================================================================
void TDlgDialog::SetupWindow()
{
   uchar buff[40];
   if(!log_pass_flag)
      CheckRadioButton(IDC_RADIOALLP, IDC_RADIOALARMP, IDC_RADIOALLP);
   else
      CheckRadioButton(IDC_RADIOALLP, IDC_RADIOALARMP, IDC_RADIOALARMP);
   if(strcmp(filen, "") == 0)
      CheckRadioButton(IDC_RADIOLOGON, IDC_RADIOLOGOFF, IDC_RADIOLOGOFF);
   else
      CheckRadioButton(IDC_RADIOLOGON, IDC_RADIOLOGOFF, IDC_RADIOLOGON);
   if(!flag_celsius)
      CheckRadioButton(IDC_FAHRENHEIT, IDC_CELSIUS, IDC_CELSIUS);   
   else
      CheckRadioButton(IDC_FAHRENHEIT, IDC_CELSIUS, IDC_FAHRENHEIT);
      
   ftoa(((float)global_time_interval/100.0), (char *)buff, 2);
   for(int i = 0; i < 4; i++)
   {
      if(buff[i] == '.')
         buff[i] = ':';
   }
   SetDlgItemText(DISPLAY_TIME_INTERVAL, (char *)buff);
}
//============================================================================
// CloseWindow
//============================================================================
void TDlgDialog::CloseWindow(int retValue)
{
   int i = IsDlgButtonChecked(IDC_RADIOLOGON);
   if(i && strlen(filen) > 0)
      log_file_open_flag = 2;
   else if((i && !strlen(filen)) || IsDlgButtonChecked(IDC_RADIOCHANGE))
      log_file_open_flag = 1; // flag for file open dialog
   else
   {
      log_file_open_flag = 0;
      strcpy(filen, "");
   }

   i = IsDlgButtonChecked(IDC_RADIOALLP);
   if((i != 0) && retValue == IDOK)
      log_pass_flag = 0;
   if(((i = IsDlgButtonChecked(IDC_RADIOALARMP)) != 0) && retValue == IDOK)
      log_pass_flag = 1;
   if(((i = IsDlgButtonChecked(IDC_FAHRENHEIT)) != 0) && retValue == IDOK)
      flag_celsius = TRUE;
   else 
      flag_celsius = FALSE;
   
   switchflag = FALSE;
      
   TWindow::CloseWindow();
   if(time_interval >= 1.00)
   {
      global_time_interval = 60;
      old_num_waits = num_waits = (uint) time_interval;
   }
   else
   {
      global_time_interval = (uint) (time_interval*100);
      num_waits = 0;
   }

}

//---------------------------------TAboutDialog---------------------------------

class TAboutDialog : public TDialog
{
   public:
      TAboutDialog(TWindow* parent, int);
   DECLARE_RESPONSE_TABLE(TAboutDialog);
};
DEFINE_RESPONSE_TABLE1(TAboutDialog, TDialog)
END_RESPONSE_TABLE;

TAboutDialog::TAboutDialog(TWindow* parent, int DlgResNum)
   :  TWindow(parent), TDialog(parent, DlgResNum)
{

}

//==============================================================================
//         TDialog for Renaming devices on one wire port
//==============================================================================

class TRenameDlg : public TDialog
{
   public:
      TRenameDlg(TWindow* parent, int);
      void SetupWindow();
      void CloseWindow(int);
   private:
   DECLARE_RESPONSE_TABLE(TRenameDlg);
};
DEFINE_RESPONSE_TABLE1(TRenameDlg, TDialog)
END_RESPONSE_TABLE;

//============================================================================
//Constructor
//============================================================================
TRenameDlg::TRenameDlg(TWindow* parent, int DlgResNum) 
   :  TWindow(parent), TDialog(parent, DlgResNum)
{
}

//============================================================================
//SetupWindow
//============================================================================
void TRenameDlg::SetupWindow()
{
   wsprintf(rom_buffer, "%2.2X %2.2X %2.2X %2.2X %2.2X %2.2X %2.2X %2.2X", rom[7], rom[6], rom[5], rom[4], rom[3], rom[2], rom[1], rom[0]);
   SetDlgItemText(ROM_ID, rom_buffer);
   Ren_ok_flag = FALSE;
} 
//============================================================================
//CloseWindow
//============================================================================
void TRenameDlg::CloseWindow(int retVal)
{
      
   if(retVal)
   {
      GetDlgItemText(NAME_EDIT, new_name, 24); 
      Ren_ok_flag = TRUE;
   }
   else
      Ren_ok_flag = FALSE;       
   alarm_on = TRUE;
     TWindow::CloseWindow();
}


                           
//==============================================================================
//Dialog to check to see if another device will be added
//==============================================================================

class TAddDlg : public TDialog
{
   public:
      TAddDlg(TWindow* parent, int);
      void SetupWindow();
      void CloseWindow(int DlgResNum);
      
   DECLARE_RESPONSE_TABLE(TAddDlg);
};
DEFINE_RESPONSE_TABLE1(TAddDlg, TDialog)
END_RESPONSE_TABLE;

//============================================================================
//Constructor
//============================================================================
TAddDlg::TAddDlg(TWindow* parent, int DlgResNum) 
   :  TWindow(parent), TDialog(parent, DlgResNum)
{

}
//============================================================================
//SetupWindow
//============================================================================
void TAddDlg::SetupWindow()
{
   add_device = FALSE;
   if(add_DS2405)
   {
      SetDlgItemText(TEXT, "Please add a DS2405 to the Microlan");
      add_DS2405 = FALSE;
   }
   else if(add_DS1820)
   {
      SetDlgItemText(TEXT, "Please add a DS1820 to the Microlan");
      add_DS1820 = FALSE;
   }
   else if(add_another_DS2405)
   {
      SetDlgItemText(TEXT, "Please add another DS2405 to the Microlan");
      add_another_DS2405 = FALSE;
   }
}
//============================================================================
//CloseWindow
//============================================================================
void TAddDlg::CloseWindow(int retVal)
{
   if(retVal)
   {
      add_device = TRUE;
   }
   else
      add_device = FALSE;
   TWindow::CloseWindow();
}


class TPreviousDlg : public TDialog
{
   public:
      TPreviousDlg(TWindow* parent, int);
      void CloseWindow(int);
   DECLARE_RESPONSE_TABLE(TPreviousDlg);
};
DEFINE_RESPONSE_TABLE1(TPreviousDlg, TDialog)
END_RESPONSE_TABLE;
//============================================================================
// constructor
//============================================================================
TPreviousDlg::TPreviousDlg(TWindow* parent, int DlgResNum)
   : TWindow(parent), TDialog(parent, DlgResNum)
{

}
//============================================================================
// CloseWindow
//============================================================================
void TPreviousDlg::CloseWindow(int retVal)
{
   if(retVal)
      previous_flag = TRUE;
   else
      previous_flag = FALSE;
   
   TDialog::CloseWindow();
}

   
//==============================================================================
// Change Temperature Thresholds for devices
//==============================================================================

class TChangeTempDlg : public TDialog
{
   public:
      TChangeTempDlg(TWindow* parent, int);
      ~TChangeTempDlg();
      void SetupWindow();
      void CloseWindow(int);
      void EvCbnSelChange();
      int i;
    private:
      TComboBox* PartList;
      void upper_temp(UINT code, UINT pos, HWND wnd);
      void EvVScroll(UINT code, UINT pos, HWND wnd);
      void lower_temp(UINT code, UINT pos, HWND wnd);
      struct tnode far * s1;
      void IdSaveButton();
      void IdFahrenheit();
      void IdCelsius();
   DECLARE_RESPONSE_TABLE(TChangeTempDlg);
};
DEFINE_RESPONSE_TABLE1(TChangeTempDlg, TDialog)
EV_CBN_SELCHANGE(ID_MYCOMBOBOX, EvCbnSelChange),
EV_WM_VSCROLL,
EV_COMMAND(IDC_SAVE_BUTTON, IdSaveButton),
//EV_COMMAND(IDC_FAHRENHEIT, IdFahrenheit),
//EV_COMMAND(IDC_CELSIUS, IdCelsius),
END_RESPONSE_TABLE;

//============================================================================
// Constructor
//============================================================================
TChangeTempDlg::TChangeTempDlg(TWindow* parent, int DlgResNum) 
   :  TWindow(parent), TDialog(parent, DlgResNum)
{
   PartList = new TComboBox(this, ID_MYCOMBOBOX); 
}

//============================================================================
// Destructor
//============================================================================
TChangeTempDlg::~TChangeTempDlg()
{
   int idevtimer = FALSE;
   int idmytimer = FALSE;
   if(PartList)
      delete PartList;
   if(ID_MYTIMER != NULL)
   {
      ::KillTimer(NULL, ID_MYTIMER);
      idmytimer = TRUE;
   }
   if(ID_EVTIMER != NULL)   
   {
      KillTimer(1);
      idevtimer = TRUE;
   }
   if(ID_TEMPTIMER != NULL)
      ::KillTimer(NULL, ID_TEMPTIMER);
   for(i = 0; i < 3000; i++);
   set_temp_parameters(root);
   if(idmytimer)
   {
      TModule *TM;
      TM = GetModule();
      ID_MYTIMER = ::SetTimer(NULL, 1, 31000, (TIMERPROC)MakeProcInstance((FARPROC)Save, TM->GetInstance())); 
   }
   if(idevtimer)   
   {
      ID_EVTIMER  = SetTimer(1, global_time_interval*1000);
   }
   finished_flag = TRUE;
}

//============================================================================
// SetupWindow
//============================================================================
void TChangeTempDlg::SetupWindow()
{
   char buff[10];
   float temp1, temp2;
   TDialog::SetupWindow();
   CheckRadioButton(UPPER_TEMP, LOWER_TEMP, UPPER_TEMP);
      
   for(i = 0; i < names->length; i++)
      PartList->AddString(names->name[i]);
   PartList->SetSelString(names->name[0], 0);
   found_string = NULL;
   s1 = search_tree_string(root, names->name[0]);
   s1 = found_string;
   if(!flag_celsius)
   {
      itoa(upper_alarm_temp, buff, 10);
      strcat(buff, " C");
      SetDlgItemText(IDC_UPPER_TEMP, buff);
      itoa(lower_alarm_temp, buff, 10);
      strcat(buff, " C");
      SetDlgItemText(IDC_LOWER_TEMP, buff);
   }
   else
   {
      temp1 = upper_alarm_temp*9/5 + 32;
      temp2 = lower_alarm_temp*9/5 + 32;

      itoa(temp1, buff, 10);
      strcat(buff, " F");
      SetDlgItemText(IDC_UPPER_TEMP, buff);

      itoa(temp2, buff, 10);
      strcat(buff, " F");
      SetDlgItemText(IDC_LOWER_TEMP, buff);    
   } 
   return;
}

//============================================================================
// CloseWindow
//============================================================================
void TChangeTempDlg::CloseWindow(int)
{
   IdSaveButton();
   TWindow::CloseWindow();   
}
//============================================================================
// function to handle the changing of the temperature
// with the scrollbar, checks if upper or lower radio is checked
// and increments approiate temperature
//============================================================================
void TChangeTempDlg::EvCbnSelChange()
{

   int i = PartList->GetSelIndex();
   if(i >= 0)
   {
      char seltext[30], upper_temp[10], lower_temp[10];  
      PartList->GetSelString(seltext, sizeof(seltext));
   
      if(s1 != NULL)
      {
         s1->maxtemp = upper_alarm_temp;
         s1->mintemp = lower_alarm_temp;   
      }
      found_string = NULL;
      s1 = search_tree_string(root, seltext);
      s1 = found_string;
      if(s1 != NULL)
      {
         if(flag_celsius)
         {
            int int_1 = s1->maxtemp * 9 / 5 + 32;
            int int_2 = s1->mintemp * 9 / 5 + 32;
            
            itoa(int_1, upper_temp, 10);
            itoa(int_2, lower_temp, 10);
            upper_alarm_temp = s1->maxtemp;
            lower_alarm_temp = s1->mintemp;
            strcat(upper_temp, " F");
            strcat(lower_temp, " F");
         }
         else
         {
            itoa(s1->maxtemp, upper_temp, 10);
            itoa(s1->mintemp, lower_temp, 10);
            upper_alarm_temp = s1->maxtemp;
            lower_alarm_temp = s1->mintemp;
            strcat(upper_temp, " C");
            strcat(lower_temp, " C");
         }
         SetDlgItemText(IDC_LOWER_TEMP, lower_temp);
         SetDlgItemText(IDC_UPPER_TEMP, upper_temp);
      }         
   }
   else
      MessageBox("Device Not Found");
}

//============================================================================
// Used to save the temperatures to the structures
//============================================================================
void TChangeTempDlg::IdSaveButton()
{
   if(s1 != NULL)
   {
      s1->maxtemp = upper_alarm_temp;
      s1->mintemp = lower_alarm_temp;
   }
}

//============================================================================
// increments the approiate temperature
//============================================================================
void TChangeTempDlg::EvVScroll(UINT code, UINT pos, HWND wnd)
{
   DestroyCaret();

   if(IsDlgButtonChecked(UPPER_TEMP) != 0)
      upper_temp(code, pos, wnd);
   else
      lower_temp(code, pos, wnd);
} 

//============================================================================
// Set the text to the temperature converted string
//============================================================================
void TChangeTempDlg::upper_temp(UINT code, UINT, HWND)
{
   char buff[40];
   int temp_alarm;
   DestroyCaret();

   switch(code)
   {
      case 0:
         upper_alarm_temp++;
         if(upper_alarm_temp > 125)
            upper_alarm_temp = 125;
         
      break;
      case 1:
         upper_alarm_temp--;
         if(upper_alarm_temp < lower_alarm_temp)
            upper_alarm_temp = lower_alarm_temp + 1; 
      break;
      default:
         break;
      }
      if(flag_celsius)
      {
         temp_alarm = upper_alarm_temp * 9 / 5 + 32;
         itoa(temp_alarm, buff, 10);
         strcat(buff, " F");
      }
      else
      {
         itoa(upper_alarm_temp, buff, 10);
         strcat(buff, " C");
      }
      SetDlgItemText(IDC_UPPER_TEMP, buff);
}
//============================================================================
// set the text to the temperature converted string
//============================================================================
void TChangeTempDlg::lower_temp(UINT code, UINT, HWND)
{
   char buff[40];
   int temp_alarm;
   switch(code)
   {
      case 0:
         lower_alarm_temp++;
         if(lower_alarm_temp > upper_alarm_temp)
            lower_alarm_temp = upper_alarm_temp - 1;
         
      break;
      case 1:
         lower_alarm_temp--;
         if(lower_alarm_temp < -55)
            lower_alarm_temp = -55;
      break;
      default:
         break;
   }
   if(flag_celsius)
   {
      temp_alarm = lower_alarm_temp * 9/5 + 32;
      itoa(temp_alarm, buff, 10);
      strcat(buff, " F");
   }
   else
   {
      itoa(lower_alarm_temp, buff, 10);
      strcat(buff, " C");
   }
   SetDlgItemText(IDC_LOWER_TEMP, buff);
}
//============================================================================
// //Dealing with fahrenheit, need to convert variable to F
//============================================================================
void TChangeTempDlg::IdFahrenheit()
{
   int temp_alarm;
   char buff[10];
   
   temp_alarm = lower_alarm_temp * 9 / 5 + 32;
   itoa(temp_alarm, buff, 10);
   strcat(buff, " F");
   SetDlgItemText(IDC_LOWER_TEMP, buff);   
   temp_alarm = upper_alarm_temp * 9 / 5 + 32;
   itoa(temp_alarm, buff, 10);
   strcat(buff, " F");
   SetDlgItemText(IDC_UPPER_TEMP, buff);
   
}
//============================================================================
// Dealing with celsius, just display the equivalent text string
//============================================================================
void TChangeTempDlg::IdCelsius()
{
   char buff[10];
   itoa(lower_alarm_temp, buff, 10);
   strcat(buff, " C");
   SetDlgItemText(IDC_LOWER_TEMP, buff);
   itoa(upper_alarm_temp, buff, 10);
   strcat(buff, " C");
   SetDlgItemText(IDC_UPPER_TEMP, buff);
}



//============================================================================
// 2405 class - class for assigning 2405s to heater or cooler
//============================================================================
class TDS2405Dlg : public TDialog
{
   public:
      TDS2405Dlg(TWindow* parent, int);
      ~TDS2405Dlg() {if(upperrad) delete upperrad; if(lowerrad) delete lowerrad;};
      void CloseWindow(int);
      void SetupWindow();
      TRadioButton *upperrad;
      TRadioButton *lowerrad;
      DECLARE_RESPONSE_TABLE(TDS2405Dlg);      
};

DEFINE_RESPONSE_TABLE1(TDS2405Dlg, TDialog)
END_RESPONSE_TABLE;

//============================================================================
// Constructor
//============================================================================
TDS2405Dlg::TDS2405Dlg(TWindow* parent, int DlgResNum)
   : TWindow(parent), TDialog(parent, DlgResNum)
{
   upperrad = 0;
   lowerrad = 0;
   //display the approiate radio buttons
   if(upper_cntl[0] == 0x00 && lower_cntl[0] == 0x00)
   {
      upperrad = new TRadioButton(this, UPPER_CNTL, "Cooler", 30, 100, 75, 20);
      lowerrad = new TRadioButton(this, LOWER_CNTL, "Heater", 132, 100, 75, 20);
      Invalidate(TRUE);
   }
   else if(upper_cntl[0] != 0 && lower_cntl[0] == 0x00)
      lowerrad = new TRadioButton(this, LOWER_CNTL, "Heater", 132, 100, 75, 20);
   else 
      upperrad = new TRadioButton(this, UPPER_CNTL, "Cooler", 30, 100, 75, 20);
}
//============================================================================
// setupwindow - displays the romid and checks the default radio button
//============================================================================
void TDS2405Dlg::SetupWindow()
{
   TDialog::SetupWindow();
   char rom_buffer[30];
   wsprintf(rom_buffer, "%2.2X %2.2X %2.2X %2.2X %2.2X %2.2X %2.2X %2.2X", rom[7], rom[6], rom[5], rom[4], rom[3], rom[2], rom[1], rom[0]);
   SetDlgItemText(ROM_CODE, rom_buffer);
   
   if(upper_cntl[0] == 0)
      CheckRadioButton(UPPER_CNTL, LOWER_CNTL, UPPER_CNTL);   
   else
      CheckRadioButton(UPPER_CNTL, LOWER_CNTL, LOWER_CNTL);
}
//============================================================================
// CloseWindow - reads the checked radio button and assigns the romid
// to the approiate function
//============================================================================
void TDS2405Dlg::CloseWindow(int)
{
   
   if(IsDlgButtonChecked(UPPER_CNTL))
   {
      strcpy(new_name, "Upper Control");
      Ren_ok_flag = TRUE;
      mmemcpy(upper_cntl, (uchar *)rom, 16);
   }
   if(IsDlgButtonChecked(LOWER_CNTL))
   {
      strcpy(new_name, "Lower Control");
      Ren_ok_flag = TRUE;
      mmemcpy(lower_cntl, (uchar *)rom, 16);
   }
   TWindow::CloseWindow();
}
