/******************************************************************************\
*  Exp4                                                                        *
*  Copyright (C) 1995 Dallas Semiconductor Corporation.                        *
*  All rights Reserved. Printed in U.S.A.                                      *
*  This software is protected by copyright laws of                             *
*  the United States and of foreign countries.                                 *
*  This material may also be protected by patent laws of the United States     *
*  and of foreign countries.                                                   *
*  This software is furnished under a license agreement and/or a               *
*  nondisclosure agreement and may only be used or copied in accordance        *
*  with the terms of those agreements.                                         *
*  The mere transfer of this software does not imply any licenses              *
*  of trade secrets, proprietary technology, copyrights, patents,              *
*  trademarks, maskwork rights, or any other form of intellectual              *
*  property whatsoever. Dallas Semiconductor retains all ownership rights.     *
\******************************************************************************/

/////////////////////////////////////////////////////////////////////
//                                                                 //
// See tree2.cpp for explanation of functions:::                   //
//                                                                 // 
//                                                                 //
/////////////////////////////////////////////////////////////////////




void mstrcpy(char far *dest, char far *src)
{
   int i = 0;

   do
   {
      dest[i] = src[i];
   }
   while (src[i++] != 0);
}
void mmemcpy(uchar far *dest, uchar far *src, uchar transnum)
{
   uchar jll;

  if (src == NULL)
    dest = NULL;
  else
    for (jll = 0; jll < transnum; jll++)
      *(dest + jll) = *(src + jll);
}
int mmemcmp(uchar far *m1, uchar far *m2, int n)
{
  int i;

  for(i=0;i<n;i++)
  {
    if(*(m1 + i) == *(m2 + i))
      continue;
      
    if(*(m1 + i) < *(m2 + i))
      return -(++i); // return negative of index+1 at which failure occured

    return ++i; // return index+1 at which failure occured
  }
  return 0;
}
struct tnode *addtree(struct tnode far *p, int rom1[8])
{
   int cond;

   if (p == NULL)
      {
         if((p = talloc()) != NULL)
         {
            mmemcpy((uchar far *) p->ROMID, (uchar far *) rom1, 8*sizeof(int));
            p->maxtemp = upper_alarm_temp;
            p->mintemp = lower_alarm_temp;
            mstrcpy(p->word, new_name);
            p->ageflag = age_flag;
            p->count = 1;
            p->set_alarm = alarm_on;
            p->left = p->right = NULL;
            strcpy(p->temperature, "");
         }
         
      }
   else if((cond = mmemcmp((uchar far *) rom1, (uchar far *) p + 25, 8*sizeof(int))) == 0)
      p->count++;
   else if (cond < 0)
      {
         p->left = addtree(p->left, rom1);
      }
   else
      {
         p->right = addtree(p->right, rom1);
      }
   return p;
}

int agetree(struct tnode *p)
{
   if (p != NULL)
   {
      agetree(p->left);
      p->alarming = FALSE;
      if(!p->ageflag && !switchflag)
      {
         strcat(szmissing, p->word);
         strcat(szmissing, " ");
         p->temperature[0] = 0;
         //return 0;
      }
      else if(p->ageflag && switchflag)
      {
         strcat(szmissing, p->word);
         strcat(szmissing, " ");
      }
   
      if(!switchflag)
         p->ageflag = 0;
      else if(switchflag)
         p->ageflag = 1;
      agetree(p->right);
   }
   if(szmissing[0] != 0)
      return 0;   
   return 1;   
}

void deltree(struct tnode *p)
{
   if(p == NULL) return;

   struct tnode far *p1;
   
   if(p->left != NULL)
   {
      p1 = p->left;
      p->left = NULL;
      deltree(p1);
   }

   if(p->right != NULL)
   {
      p1 = p->right;
      p->right = NULL;
      deltree(p1);
   }

   delete(p);
   return;
}

int set_temp_parameters(struct tnode *p)
{
   changing_temp = TRUE;
   if(!TMValidSession(handle))
   {
      TMEndSession(handle);
      handle = TMStartSession(comport);
      TMSetup(handle);
   }

   if (p != NULL)
   {
      set_temp_parameters(p->left);
      if(p->set_alarm)
      {
         TMRom(handle, GB, p->ROMID);
         TMAccess(handle, GB);
         TMTouchByte(handle, 0x4E);
         TMTouchByte(handle, p->maxtemp);
         TMTouchByte(handle, p->mintemp);
         TMTouchReset(handle);
         TMAccess(handle, GB);
         TMTouchByte(handle, 0x48);
         for(int i=0; i < 500; i++){};
      }
      set_temp_parameters(p->right);
   }
   changing_temp = FALSE;
   return 1;
   
}

int agetreenull(struct tnode *p)
{
   if (p != NULL)
   {
      agetreenull(p->left);
      p->ageflag = 0;
      agetreenull(p->right);
   }
   return TRUE;
}


struct tnode *search_tree(struct tnode *p, int rom1[8])
{
   if(!p) return p;
   int cond;
   while ((cond = mmemcmp((uchar *) p->ROMID, (uchar *) rom1, 16)) != 0)
   {
      if(cond > 0) p = p->left;
      else p = p->right;
      if(p == NULL)
         break;
   }
   if(cond == 0 && p != NULL && !switchflag)
      p->ageflag = 1;
   else if(cond == 0 && p != NULL && switchflag)
      p->ageflag = 1;
   return p;
}
struct tnode *search_tree_string(struct tnode *p, char *pstring)
{
   if(!p) return p;
   if (p != NULL)
   {
      search_tree_string(p->left, pstring);
      if(strcmp(p->word, pstring) == 0)
      {
         found_string = p;
         return p;
      }                     
      search_tree_string(p->right, pstring);
   }
   return p;
   
}
struct tnode far *talloc()
{
   return new struct tnode;
}

int Check_Tree()
{
//   struct tnode far *searchroot; 
   rom[0] = 0x00;
   if(!TMValidSession(handle))
   {
      TMEndSession(handle);
      handle = TMStartSession(comport);
      TMSetup(handle);
   }

   if(!switchflag)
   {
      if(TMFirst(handle, GB))
      {
         TMRom(handle, GB, rom);
         search_tree(root, rom);
      }
      while(TMNext(handle, GB))
      {
         rom[0] = 0x00;
         TMRom(handle, GB, rom);
         search_tree(root, rom);
      }   
   
      return(agetree(root));
   }
   else
   {
      agetreenull(root);
      if(TMFirst(handle, GB))
      {
         TMRom(handle, GB, rom);
         search_tree(root, rom);
      }
      else
         return (agetree(root));
      while(TMNext(handle, GB))
      {
         rom[0] = 0x00;
         TMRom(handle, GB, rom);
         search_tree(root, rom);
      }   
   
      return(agetree(root));
   }   
}

int alarm_search()
{
   int alarm_rom[8], i, found= FALSE;
   struct tnode *struct1;

   if(!TMValidSession(handle))
   {
      TMEndSession(handle);
      handle = TMStartSession(comport);
      TMSetup(handle);
   }

   i = TMFirstAlarm(handle, GB);
   if(i != 0)
   {
      
      alarm_rom[0] = 0;
      TMRom(handle, GB, alarm_rom);
      struct1 = search_tree(root, alarm_rom);
      if(struct1 != NULL)
      {
         alarm_flag = TRUE;
         struct1->alarming = TRUE;
           strcpy(alarming_devices, struct1->word);
           strcat(alarming_devices, " ");
           strcat(alarming_devices, struct1->temperature);
         found = TRUE;
      }
      else
         found = FALSE;
      while(TMNextAlarm(handle, GB))
      {
         alarm_rom[0] = 0;
         TMRom(handle, GB, alarm_rom);
         struct1 = search_tree(root, alarm_rom);
         if(struct1 != NULL)
         {
            struct1->alarming = TRUE;
            strcat(alarming_devices, " ");
            strcat(alarming_devices, struct1->word);
            strcat(alarming_devices, " ");
            strcat(alarming_devices, struct1->temperature);
         }
      }
   }
   return found;   
}

void copy_names(struct tnode *p)
{
   if (p != NULL)
   {
      copy_names(p->left);
      
      if(p->ROMID[0] == 0x10)
      {
         mstrcpy(names->name[names->length], p->word);
         names->length++;
      }
         
      copy_names(p->right);
   }
   return;      
}

void save_setup1(struct tnode *p)
{
   if (p!=NULL)
   {
      save_setup1(p->left);
         write(out_stream, (void *) p, sizeof(tnode));
      save_setup1(p->right);
   }
   return;
}
void save_setup(struct tnode *p)
{
   write(out_stream, (void *)&comport, sizeof(comport));
   write(out_stream, (void *)&flag_celsius, sizeof(flag_celsius));
   write(out_stream, (void *)&switchflag, sizeof(switchflag));
   write(out_stream, (void *)&log_pass_flag, sizeof(log_pass_flag));
   write(out_stream, (void *)&global_time_interval, sizeof(global_time_interval));
   write(out_stream, (void *)filen, 80);
   write(out_stream, (void *)&log_file_open_flag, sizeof(log_file_open_flag));
   write(out_stream, (unsigned char *)upper_cntl, 16);
   write(out_stream, (unsigned char *)lower_cntl, 16);
   save_setup1(p);
   write(out_stream, (void *)&num_devices, sizeof(num_devices));  
}
void read_settings1(void)
{
   struct tnode *p;
   num_devices = 0;
   p = new tnode;
   if(in_stream == NULL)
      return;
   while(read(in_stream, (void *) p, sizeof(tnode)) == sizeof(tnode))
   {  
      upper_alarm_temp = p->maxtemp;
      lower_alarm_temp = p->mintemp;
      mstrcpy(new_name, p->word);
      age_flag = p->ageflag;
      alarm_on = p->set_alarm;
      root = addtree(root, p->ROMID);
      /*if(root == NULL)
      {
         upper_alarm_temp = p->maxtemp;
         lower_alarm_temp = p->mintemp;
         mstrcpy(new_name, p->word);
         age_flag = p->ageflag;
         alarm_on = p->set_alarm;
        //   root = addtree(root, p->ROMID);
      }
      else
      {
         upper_alarm_temp = p->maxtemp;
         lower_alarm_temp = p->mintemp;
         mstrcpy(new_name, p->word);
         age_flag = p->ageflag;
         alarm_on = p->set_alarm;
        //   addtree(root, p->ROMID);
      }      */
   }
   close(in_stream);
   return;
}
void read_settings(void)
{
   read(in_stream, (void *)&comport, sizeof(comport));
   read(in_stream, (void *)&flag_celsius, sizeof(flag_celsius));
   read(in_stream, (void *)&switchflag, sizeof(switchflag));
   read(in_stream, (void *)&log_pass_flag, sizeof(log_pass_flag));
   read(in_stream, (void *)&global_time_interval, sizeof(global_time_interval));
   read(in_stream, (void *)filen, 80);
   read(in_stream, (void *)&log_file_open_flag, sizeof(log_file_open_flag));
   read(in_stream, (unsigned char *)upper_cntl, 16);
   read(in_stream, (unsigned char *)lower_cntl, 16);
   read_settings1();
   read(in_stream, (void *)&num_devices, sizeof(num_devices));
}
uchar SwitchState(uchar far *RI)
{
   if(!TMValidSession(handle))
   {
      handle = TMStartSession(comport);
      TMSetup(handle);
   }

   TMRom(handle, GB, (int *)RI);
   return TMStrongAlarmAccess(handle, GB);
}

void SwitchOn(uchar far *RI)
{
   ulong wait;
   if(!finished_flag)
      return;      
   if(!TMValidSession(handle))
   {
      TMEndSession(handle);
      handle = TMStartSession(comport);
      TMSetup(handle);
   }

   if(!SwitchState(RI))
   {
      
      TMAccess(handle, GB);
      wait_counts(2400L);
   }
   return;
}

void SwitchOff(uchar far *RI)
{
   if(!finished_flag)
      return;

   if(!TMValidSession(handle))
   {
      TMEndSession(handle);
      handle = TMStartSession(comport);
      TMSetup(handle);
   }

   if(SwitchState(RI))
   {
      TMAccess(handle, GB);
   }
   return;
}

static void wait_counts(ulong wait_count)
{
   unsigned short icount, ccount = 0, lcount;

   outp(0x43, 0);                              // Freeze the initial count 
   lcount = inp(0x40);                           // Read low byte of count 
   icount = (inp(0x40) << 8) | lcount;           // Read the initial count 
   do
   {
      outp(0x43, 0);                           // Freeze the current count 
      lcount = inp(0x40);                        // Read low byte of count 
      ccount = (inp(0x40) << 8) | lcount;        // Read the initial count 
   }                                                                       
   while ((unsigned short) (icount - ccount) < wait_count);          // Burn cycles
   
}
///////////////////////////////////////////////////////////////////////
char *ftoa(double d, char *buff, int numdecplaces)
{
  int i;
  long ltemp;
  char tempbuff[80];
  double mult;


  buff[0] = '\0';
  mult = 100.00;
  ltemp = (long)(mult*d);
// get integer part
  strcpy(buff,ltoa(ltemp/ (long) mult,tempbuff,10));
  strcat(buff,".");
// get decimal part
  ltemp = labs(ltemp);
  ltemp %= (long)mult;

  for (i=0;i<(numdecplaces-1);i++)
  {
    if (ltemp < pow10(i+1))
      strcat (buff,"0");
  }
  strcat(buff,ltoa(ltemp,tempbuff,10));
  return buff;
}
///////////////////////////////////////////////////////////////////////

