/******************************************************************************\
*  Exp5                                                                        *
*  Copyright (C) 1995 Dallas Semiconductor Corporation.                        *
*  All rights Reserved. Printed in U.S.A.                                      *
*  This software is protected by copyright laws of                             *
*  the United States and of foreign countries.                                 *
*  This material may also be protected by patent laws of the United States     *
*  and of foreign countries.                                                   *
*  This software is furnished under a license agreement and/or a               *
*  nondisclosure agreement and may only be used or copied in accordance        *
*  with the terms of those agreements.                                         *
*  The mere transfer of this software does not imply any licenses              *
*  of trade secrets, proprietary technology, copyrights, patents,              *
*  trademarks, maskwork rights, or any other form of intellectual              *
*  property whatsoever. Dallas Semiconductor retains all ownership rights.     *
\******************************************************************************/

//==============================================================================
// Child Class:  Only one child window is allowed at a time.  SDI would have 
// worked.
//==============================================================================
class TDecoChild: public TMDIChild {
public:
   time_t timer;
   struct tm *tblock;   
   TDecoChild(TMDIClient& parent, const char far *name);
   ~TDecoChild();
   bool CanClose();
   void SetupWindow();
   int Unnamed() { return filename.length() == 0; }
   int SaveFile();
   void EvTimer(UINT timerId);
   void Paint(TDC& dc, BOOL, TRect&);
   int RomID[8];
   int numRows, numCols;
   double temp10;
   string filename;     //used to store the logfile name;
   TIArrayAsVector<string> sArray; //array for devices.

private:
   int cyChar;
   long width, height;
   int rowSep, colSep;
   TSize increment;
   uchar first_pass;
//   TIArrayAsVector<string> s1Array;//array for temperatures.
   DECLARE_RESPONSE_TABLE(TDecoChild);
};

// Default constructor for File|New
DEFINE_RESPONSE_TABLE1(TDecoChild, TMDIChild)
   EV_WM_TIMER,
END_RESPONSE_TABLE;

TDecoChild::TDecoChild(TMDIClient& parent, const char far *name)
  : TMDIChild(parent, name), sArray(SARRAY_HIGH, SARRAY_LOW, SARRAY_DELTA)
  //s1Array(SARRAY_HIGH, SARRAY_LOW, SARRAY_DELTA)
{
   filename = name;
   Attr.Style |= MDIS_ALLCHILDSTYLES;
   Attr.Style &= ~WS_MAXIMIZEBOX;
   Attr.Style &= ~WS_MINIMIZEBOX;
   Attr.Style |=WS_MAXIMIZE;
   Scroller = new TScroller(this, 0, 0, 0, 0);
   numRows = numCols = 0;
   rowSep = colSep = 0;
   first_pass = TRUE;
   numRows = 0;
      
   TModule *TM;
   TM = GetModule();

   // timer for saving file every few minutes.  time controlled by a define statement;
   if(strcmp(name, "") != 0)
      ID_MYTIMER = ::SetTimer(NULL, 1, 30000, (TIMERPROC)MakeProcInstance((FARPROC)Save, TM->GetInstance())); 
}
TDecoChild::~TDecoChild()
{
   if(timer_set_flag)         //kill all timers and make sure 2405's off
   {
      KillTimer(ID_EVTIMER);
      timer_set_flag = FALSE;
      ID_EVTIMER = 0;
   }
   if(ID_TEMPTIMER)
   {
      ::KillTimer(NULL, ID_TEMPTIMER);
      ID_TEMPTIMER = 0;
   }
   if(ID_MYTIMER != NULL)
   {
      ::KillTimer(NULL, ID_MYTIMER);
      timecount = 0;
      cp->SaveFile();
      ID_MYTIMER = 0;
   }
   SwitchOff(upper_cntl);
   SwitchOff(lower_cntl);
   SwitchOff(alarm_cntl);

   return;
}

bool TDecoChild::CanClose()
{
   return FALSE;
}

// function used to setup autosave flag;
WORD FAR PASCAL Save(HWND, WORD, WORD, LONG)
{
   cp->SaveFile();
   return 0;
}
// save the log file;
int TDecoChild::SaveFile()
{
   char c[80];
   const char* cst;
   if(timecount == 0)
   {
      //check to make sure file needs to be saved.
      if(currentRows != numRows)
      {
         FILE *fp;
         fp = fopen(filename.c_str(), "a+");
         for(int i = currentRows; i < numRows; i++)   
         {   
            cst = sArray[i]->c_str();
            strcpy(c, cst);
            int len;
            c[(len = strlen(c))] = 0x0a;
            c[len+1] = 0;
            fwrite(c, strlen(c), 1, fp);
         }
         currentRows = numRows;
         timecount = WAIT_TIME;
         fclose(fp);
      }
      else
         timecount = WAIT_TIME;
   }
   else
      --timecount;
   return TRUE;
}
void TDecoChild::EvTimer(UINT)
{
   char s[640], temp_string[500];
   int j;

   temp_string[0] = 0;
   
   if(!finished_flag)
      return;
   if(kill_timer_flag)
   {
      KillTimer(NULL);
      ::KillTimer(NULL, ID_MYTIMER);
      return;
   }
   if(num_waits == 0)
   {
      szmissing[0] = 0;
      
      if(Check_Tree())
      {

         if(!TMValidSession(handle))
         {
            TMEndSession(handle);
            handle = TMStartSession(comport);
            TMSetup(handle);
         }
//         SwitchOff(alarm_cntl);

         TMTouchReset(handle);

         memcpy((uchar *)rom, (uchar *)temp_rom, 16);
         TMTouchByte(handle, 0x55);
         TMRom(handle, GB, rom);
         TMAccess(handle, GB);
         TMTouchByte(handle, 0x44);
         finished_flag = FALSE;
         TModule *TM;
         TM = GetModule();
         ID_TEMPTIMER = ::SetTimer(NULL, 2, 1000, (TIMERPROC)MakeProcInstance((FARPROC)Temperature, TM->GetInstance()));

         //determine which logging procedure is in use.
         if(!log_pass_flag && !alarm_flag || (log_pass_flag && first_pass && !alarm_flag))
         {
            first_pass = FALSE;
            numRows++;
            strcpy(s, sz);
            strcat(s, " ");
            timer = time(NULL);
            tblock = localtime(&timer);
            strcat(s, asctime(tblock));
            s[strlen(s) - 1] = 0;
            sArray.Add(new string(s));
         }
         else if(alarm_flag)
         {
            numRows++;
            strcpy(s, alarming_devices);
            alarming_devices[0] = 0;    //clear it for next pass
            strcat(s, " ");
            timer = time(NULL);
            tblock = localtime(&timer);
            strcat(s, asctime(tblock));
            s[strlen(s) - 1] = 0;
            strcat(s, " ");
            alarm_flag = FALSE;
            first_pass = TRUE;
            if(flag_celsius)
            {
               if(temp10 > (upper_alarm_temp*9/5 + 32) )
                  strcat(s, " Cooler ");
               else if(temp10 < (lower_alarm_temp*9/5 + 32))
                  strcat(s, " Heater ");
            }        
            else 
            {
               if(temp10 > upper_alarm_temp)
                  strcat(s, " Cooler");
               else if(temp10 < lower_alarm_temp)
                  strcat(s, " Heater");
            }
            sArray.Add(new string(s));
            alarm_flag = FALSE;
         } 
         sz[0] = 0;
      }
      else
      {
         if(!TMValidSession(handle))
         {
            TMEndSession(handle);
            handle = TMStartSession(comport);
            TMSetup(handle);
         }
         if(alarm_enable)    //check to see if alarm is enabled
            SwitchOn(alarm_cntl);
         TMTouchReset(handle);

         memcpy((uchar *)rom, (uchar *)temp_rom, 16);
         TMTouchByte(handle, 0x55);
         TMRom(handle, GB, rom);
         TMAccess(handle, GB);
         TMTouchByte(handle, 0x44);
         finished_flag = FALSE;
         TModule *TM;
         TM = GetModule();
         ID_TEMPTIMER = ::SetTimer(NULL, 2, 1000, (TIMERPROC)MakeProcInstance((FARPROC)Temperature, TM->GetInstance()));

         numRows++;
         numRows++;
         strcpy(s, sz);
         strcat(s, " ");
         timer = time(NULL);
         tblock = localtime(&timer);
         strcat(s, asctime(tblock));
         s[strlen(s) - 1] = 0;
         if(flag_celsius)
         {
            if(temp10 > (upper_alarm_temp*9/5 + 32) )
               strcat(s, "  Cooler ");
            else if(temp10 < (lower_alarm_temp*9/5 + 32))
               strcat(s, "  Heater ");
         }        
         else 
         {
            if(temp10 > upper_alarm_temp)
               strcat(s, "  Cooler");
            else if(temp10 < lower_alarm_temp)
               strcat(s, "  Heater");
         }
         sArray.Add(new string(s));

         strcpy(s, "missing or open: ");
         strcat(s, szmissing);
         sArray.Add(new string(s));
         sz[0] = 0;
         first_pass = TRUE;
      }
      Invalidate(TRUE);
      num_waits = old_num_waits;
  }
  else
   num_waits--;
}

void TDecoChild::SetupWindow()
{
   TFrameWindow::SetupWindow();
   TEXTMETRIC tm;
   dc = new TClientDC(*this);
   dc->GetTextMetrics(tm);
   cyChar = tm.tmHeight + tm.tmExternalLeading;
   delete dc;

   ID_EVTIMER = SetTimer(1, global_time_interval*1000);
   timer_set_flag = TRUE;

   TMTouchReset(handle);
   memcpy((uchar *)rom, (uchar *)temp_rom, 16);
   TMTouchByte(handle, 0x55);
   TMRom(handle, GB, rom);
   TMAccess(handle, GB);
   TMTouchByte(handle, 0x44);
   finished_flag = FALSE;
   TModule *TM;
   TM = GetModule();
   ID_TEMPTIMER = ::SetTimer(NULL, 2, 1000, (TIMERPROC)MakeProcInstance((FARPROC)Temperature, TM->GetInstance()));

   return;
}

// used to print out sArray onto the screen
void TDecoChild::Paint(TDC& dc, BOOL, TRect&)
{
   TRect r;
   int row, x, y;
   const char* c;
   int linenum = 0;
   if(numRows == 0) return;
   dc.GetClipBox(r);
   if(numRows < 20)
      row = 0;
   else 
      row = numRows - 20; 
   for (row; row < numRows; ++row)
   {
      x = colSep;
      y = linenum * cyChar;
      linenum++;
      c = sArray[row]->c_str();
      dc.TextOut(x, y, c, strlen(c));
   }
}

//ev_timer function used to measure temperature of all devices on the one-wire;
//while this function is in use, the TDecoChild::EvTimer function cannot take place;
WORD FAR PASCAL Temperature(HWND, WORD, WORD, LONG)
{
   char buf[10], buffer[60];
   int scratchpad[8];
   tnode *struct1;
   if(!changing_temp)
   {
      if(!TMValidSession(handle))
      {
         TMEndSession(handle);
         handle = TMStartSession(comport);
         TMSetup(handle);
      }
      TMTouchByte(handle, 0x55);
      TMRom(handle, GB, rom);  //rom id of 1820/1920
      TMAccess(handle, GB);    
      TMTouchByte(handle, 0xBE); //issue read scratchpad
      for(int i = 0; i < 9; i++)
         scratchpad[i] = TMTouchByte(handle, 0xFF);

      cp->temp10 = (double)(scratchpad[0] >> 1);
      if(scratchpad[0] & 0x01)
         cp->temp10 = cp->temp10 + 0.5;
      if(flag_celsius)
         cp->temp10 = cp->temp10 * 9.0 / 5.0 + 32.0;
            
      ftoa(cp->temp10, buf, 1); 
      if(!flag_celsius)
         strcat(buf, "C");
      else
         strcat(buf, "F");
      struct1 = search_tree(root, rom);                  
      
      if(scratchpad[1] == 0xFF && struct1 != NULL)
      {
         mstrcpy(struct1->temperature, "-");
         strcat(struct1->temperature, buf);
      }
      else if(struct1 != NULL)
         mstrcpy(struct1->temperature, buf);

      if(struct1 != NULL)
         strcpy(buffer, struct1->word);
      strcat(buffer, " ");
      strcat(buffer, buf);
      strcat(sz, buffer);
      first_time = FALSE;
      ::KillTimer(NULL, ID_TEMPTIMER);
      alarm_search();
      finished_flag = TRUE;
      if(flag_celsius)
      {
         if(cp->temp10 > ((double)upper_alarm_temp*9.0/5.0 + 32.0))
         {
            SwitchOn(upper_cntl);
            SwitchOff(lower_cntl);
         }
         else if(cp->temp10 < ((float)lower_alarm_temp*9/5 + 32))
         {
            SwitchOn(lower_cntl);
            SwitchOff(upper_cntl);
         }
         else
         {
            SwitchOff(lower_cntl);
            SwitchOff(upper_cntl);
         }
      }        
      else if(!flag_celsius)
      {
         if(cp->temp10 > upper_alarm_temp)
         {
            SwitchOn(upper_cntl);
            SwitchOff(lower_cntl);
         }
         else if(cp->temp10 < lower_alarm_temp)
         {
            SwitchOn(lower_cntl);
            SwitchOff(upper_cntl);
         }
         else
         {
            SwitchOff(lower_cntl);
            SwitchOff(upper_cntl);
         }
      }
   }
   return 0;
}


