/******************************************************************************\
*  Exp5                                                                        *
*  Copyright (C) 1995 Dallas Semiconductor Corporation.                        *
*  All rights Reserved. Printed in U.S.A.                                      *
*  This software is protected by copyright laws of                             *
*  the United States and of foreign countries.                                 *
*  This material may also be protected by patent laws of the United States     *
*  and of foreign countries.                                                   *
*  This software is furnished under a license agreement and/or a               *
*  nondisclosure agreement and may only be used or copied in accordance        *
*  with the terms of those agreements.                                         *
*  The mere transfer of this software does not imply any licenses              *
*  of trade secrets, proprietary technology, copyrights, patents,              *
*  trademarks, maskwork rights, or any other form of intellectual              *
*  property whatsoever. Dallas Semiconductor retains all ownership rights.     *
\******************************************************************************/

#include <owl\radiobut.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <io.h>
#include <fcntl.h>     //fopen defines
#include <sys\stat.h>
#include <owl\applicat.h>    //TApplication
#include <owl\framewin.h>    //TFramewindow
#include <owl\dc.h>          //device context
#include <owl\dialog.h>      //dialog
#include <owl\opensave.h>    //dialog
#include <windows.h>
#include <owl\scroller.h>    //scroll bars
#include <classlib\arrays.h> //TIArray
#include <cstring.h>        //TIArray
#include <owl\gdiobjec.h>   //font
#include <stdio.h>
#include <owl\controlb.h>   //control bar
#include <owl\gadget.h>     //button gadgets
#include <owl\buttonga.h>   //button gadgets
#include <owl\messageb.h>   //message bar
#include <owl\statusba.h>
#include <time.h>
#include <bwcc.h>
#include <owl\decmdifr.h>   //decorated mdiframe
#include <owl\mdi.h>
#include <owl\mdichild.h>
#include <mem.h>
#include <owl\scrollba.h>
#include <owl\combobox.h>
#include <owl\owlpch.h>
#pragma  hdrstop



#define MAXTIMEINTERVAL 60.0
#define MINTIMEINTERVAL 0.0
#define MAX_LINES 48
#define SARRAY_HIGH 249
#define SARRAY_LOW 0
#define SARRAY_DELTA 125
#define TICK_SEG 0x40
#define TICK_OFF 0x6c
#define TRUE 1
#define FALSE 0

#define LINE1 "Temperature Control Test Program"
#define LINE2 ""
#define LINE3 "Dallas Semiconductor"
#define LINE4 "Copyright 1995"
#define WAIT_TIME 10


#include "define5.h"
#include "exp5.h"
#include "tree5.cpp"
#include "dialog5.cpp"
#include "child5.cpp"



// ===========================================================
// The MDI client window class
// ===========================================================

class TMicroLanClient: public TMDIClient {
public:

   TMicroLanClient(): TMDIClient() {};
   ~TMicroLanClient() {deltree(root);}; //clean up the binary tree;
   
   
  TOpenSaveDialog::TData *data;
  
protected:
   void CmFileOpen();
   void CmHelpAbout();
   void CmLogOn();
   void CmLogOff();
   void CmSystem();
   void CmDiagnostics();
   void CmExit();
   void CmComSetup();
   void CmAlarmOn();
//   void CmAlarmOff();
   void SetupWindow();
   void Add_Device();
   void CmChange_Temp();
DECLARE_RESPONSE_TABLE(TMicroLanClient);
};

DEFINE_RESPONSE_TABLE1(TMicroLanClient, TMDIClient)
  EV_COMMAND(CM_HELPABOUT, CmHelpAbout),
  EV_COMMAND(CM_LOGSETUP, CmSystem),
  EV_COMMAND(CM_LOGON, CmLogOn),
  EV_COMMAND(CM_LOGOFF, CmLogOff),
  EV_COMMAND(CM_EXIT, CmExit),
  EV_COMMAND(CM_COMSETUP, CmComSetup),
  EV_COMMAND(CM_DIAGNOSTICS, CmDiagnostics),
  EV_COMMAND(CM_CHANGE_TEMP, CmChange_Temp),
  EV_COMMAND(CM_ALARMON, CmAlarmOn),
//  EV_COMMAND(CM_ALARMOFF, CmAlarmOff),
END_RESPONSE_TABLE;

void TMicroLanClient::SetupWindow()
{
   TWindow::SetupWindow();
     
   int k;
   names = new name_list; 
   names->length = 0;
   if(!comport)
   {
      for(int i = 1; i < 5; i++)
      {
         handle = TMStartSession(i);
         k = TMSetup(handle);
         if(k == 1)
         {
            comport = i;
            TMTouchReset(handle);
            break;
         }
      }
   }
   else
   {
      handle = TMStartSession(comport);
      k = TMSetup(handle);
   }
   TDialog* TPrev = new TPreviousDlg(this, PREVIOUS);
   TPrev->Execute();
   if(previous_flag)
   {
      in_stream = open("exp5.stp", O_RDWR | O_BINARY, S_IREAD | S_IWRITE);
      if(in_stream < 1)
           Add_Device();
      else
      {      
          read_settings();
          agetree(root);
          set_temp_parameters(root);
          copy_names(root);
      }
   }
   else
   {
      exit1= FALSE;
      Add_Device();
      
   }
   if(exit1)
      PostQuitMessage(1);
   AlarmGadget->SetButtonState(2);
}

void TMicroLanClient::Add_Device()
{
   
   int found_one;
   root = NULL;
   rom[0] = 0x00;

   TDialog* TRen = new TRenameDlg(this, REN_ID);
   TDialog* TAdd = new TAddDlg(this, ADD_ANOTHER);
   TDialog* TRen2401 = new TRenameDlg2401(this, DIALOG_3);
   struct tnode far * struct1;
   age_flag = 1; //default for the add_tree function
   add_device = FALSE;
   while(!add_device)
   {
      if(!TMValidSession(handle))
      {
         handle = TMStartSession(comport);
         TMSetup(handle);
      }
      if(TMFirst(handle, GB))
      {
         rom[0] = 0;
         TMRom(handle, GB, rom);
         if(rom[0] == 0x06)
         {
            mmemcpy((uchar *)cntl_1993, (uchar *)rom, 16);
            MessageBeep(1);
            add_device = TRUE;
         }
      }
      while(TMNext(handle, GB))
      {
         rom[0] = 0;
         TMRom(handle, GB, rom);
         if(rom[0] == 0x06)
         {
            mmemcpy((uchar *)cntl_1993, (uchar *)rom, 16);
            MessageBeep(1);
            add_device = TRUE;
         }
      }
      if(add_device)
         break;
      else
      {
         if(BWCCMessageBox(NULL, "No DS1993 present, Retry or Cancel to exit program", 
                  "Error", MB_RETRYCANCEL) != IDRETRY)
         {
            exit1 = TRUE;
            return;
         }
      }
   }
   add = 0;  //add a 2405 

   TAdd->Execute();

   if(!TMValidSession(handle))        //make sure good session
   {
      handle = TMStartSession(comport);
      TMSetup(handle);
   }
   add_device = TRUE;
 
   while(num_devices < 4 && add_device)
   {
      if(!TMValidSession(handle))
      {
         TMEndSession(handle);
         handle = TMStartSession(comport);
         TMSetup(handle);
      }
      if(TMFirst(handle, GB))
      {
         rom[0] = 0x00;
         TMRom(handle, GB, rom);
         if(rom[0] == 0x05)
         {
            struct1 = search_tree(root, rom);
            if(struct1 == NULL)
            {
               Ren_ok_flag = FALSE;
               TDialog* TDS2405 = new TDS2405Dlg(this, DS2405DLG);
               TDS2405->Execute();
               if(Ren_ok_flag)
               {
                  root = addtree(root, rom);
                  TAdd->Execute();
               }
               delete TDS2405;
               num_devices++;
               if(num_devices > 1)
                  add_device = FALSE;
               if(!add_device)
                  break;
            }
         }
         if(!add_device && num_devices > 1)
            break;
         else
            add_device = TRUE;
      }
      if(!TMValidSession(handle))
      {
         TMEndSession(handle);
         handle = TMStartSession(comport);
         TMSetup(handle);
         TMFirst(handle, GB);
      }
      while(TMNext(handle, GB) && num_devices < 3)
      {
         rom[0] = 0x00;
         TMRom(handle, GB, rom);
         if(rom[0] == 0x05)
         {
            struct1 = NULL;
            struct1 = search_tree(root, rom);
            if(struct1 == NULL)
            {
               Ren_ok_flag = FALSE;
               TDialog* TDS2405 = new TDS2405Dlg(this, DS2405DLG);
               TDS2405->Execute();
               if(Ren_ok_flag)
               {
                  num_devices++;
                  if(num_devices > 2)
                     add_device = FALSE;
                  delete TDS2405;
                  root = addtree(root, rom);
                  if(num_devices < 3)
                  {
                     add = 2;
                     TAdd->Execute();
                  }
                  else
                     add_device = FALSE;
               }   
               if(!add_device && num_devices > 1)
                  break;
               else add_device = TRUE;
            }
         }
         if(!TMValidSession(handle))
         {
            TMEndSession(handle);
            handle = TMStartSession(comport);
            TMSetup(handle);
            TMFirst(handle, GB);
         }
      }
      if(num_devices > 2)
      {
         add_device = FALSE;
         break;
      }
      
   }
   deltree(root);
   root = NULL;
   add = 1;
   TAdd->Execute();
   num_devices = 3;
   if(add_device)
   {
      if(!TMValidSession(handle))
      {
         TMEndSession(handle);
         handle = TMStartSession(comport);
         TMSetup(handle);
      }
      if(TMFirst(handle, GB))
      {
         rom[0] = 0x00;
         TMRom(handle, GB, rom);
         if(rom[0] == 0x10)
         {
            struct1 = search_tree(root, rom);
            if(struct1 == NULL)
            {
               Ren_ok_flag = FALSE;
               TRen->Execute();
               if(Ren_ok_flag)
               {
                  //root = NULL;
                  root = addtree(root, rom);
                  mmemcpy((uchar *)temp_rom, (uchar *)rom, 16);
                  num_devices++;
               }
            }
         }
      }
      while(num_devices < 4 && add_device)
      {
         if(!TMValidSession(handle))
         {
            TMEndSession(handle);
            handle = TMStartSession(comport);
            TMSetup(handle);
         }
         while(TMNext(handle, GB) && num_devices < 4)
         {
            rom[0] = 0x00;
            TMRom(handle, GB, rom);
            if(rom[0] == 0x10)
            {
               struct1 = search_tree(root, rom);
               if(struct1 == NULL)
               {
                  Ren_ok_flag = FALSE;
                  TRen->Execute();
                  if(Ren_ok_flag)
                  {
                     //root = NULL;
                     mmemcpy((uchar *)temp_rom, (uchar *)rom, 16);
                     add_device = FALSE;
                     if(!root)
                        addtree(root, rom);
                     else
                        root = addtree(root, rom);
                     num_devices++;
                  }
               }
            }
            if(!TMValidSession(handle))
            {
               TMEndSession(handle);
               handle = TMStartSession(comport);
               TMSetup(handle);
               TMFirst(handle, GB);
            }
         }   
      } 
   }
  
   add = 3;  //add a 2401 
   TAdd->Execute();
   if(!TMValidSession(handle))
   {
      TMEndSession(handle);
      handle = TMStartSession(comport);
      TMSetup(handle);
   }
   if(TMFirst(handle, GB) && add_device)
   {
      TMRom(handle, GB, rom);
      if(rom[0] == 0x01)
      {
         Ren_ok_flag = FALSE;
         TRen2401->Execute();
         if(Ren_ok_flag)
         {
            num_devices++;
            addtree(root, rom);
            add = 3;      //add another 2401
            TAdd->Execute();
         }
      }
   }
   while(add_device && num_devices < 24)
   {
      while(TRUE)
      {
         if(!TMValidSession(handle))
         {
            TMEndSession(handle);
            handle = TMStartSession(comport);
            TMSetup(handle);
         }
         found_one = FALSE;
         if(TMFirst(handle, GB) && add_device)
         {
            TMRom(handle, GB, rom);
            if(rom[0] == 0x01)
            {
               struct1 = search_tree(root, rom);
               if(struct1 == NULL)
               {
                  Ren_ok_flag = FALSE;
                  TRen2401->Execute();
                  if(Ren_ok_flag)
                  {
                     found_one = TRUE;
                     num_devices++;
                     addtree(root, rom);
                     add = 3;      //add another 2401
                     TAdd->Execute();
                  }
               }
            }
         }
         while(TMNext(handle, GB))
         {
            rom[0] = 0;
            TMRom(handle, GB, rom);
            if(rom[0] == 0x01)
            {  
               struct1 = search_tree(root, rom);
               if(struct1 == NULL)
               {
                  Ren_ok_flag = FALSE;
                  found_one = TRUE;
                  TRen2401->Execute();
                  if(Ren_ok_flag)
                  {
                     num_devices++;
                     addtree(root, rom);
                     add = 3;
                     TAdd->Execute();
                     break;
                  } //Ren_ok_flag
               }  //struct1 == NULL
               if(!add_device || num_devices > 24)
                  break;
            } //rom[0] == 0x01:
            if(!TMValidSession(handle) && add_device)
            {
               TMEndSession(handle);
               handle = TMStartSession(comport);
               TMSetup(handle);
               TMFirst(handle, GB);
            }
         } //while(next)
         if(!found_one)
         {
            add_device = FALSE;
            break;
         }
         else
            found_one = FALSE;
      }
      if(!add_device || num_devices > 24)
         break;
   }
   agetree(root);
   set_temp_parameters(root);
   copy_names(root);
   return;             

}
// Open an existing file for File|Open command
void TMicroLanClient::CmFileOpen()
{
  cp = new TDecoChild(*this, filen);
  cp->Create();
}

void TMicroLanClient::CmSystem()
{
   if(check_1993() || !alarm_enable)
   {
      TDlgDialog TD(this, DIALOG_2);
      TD.Execute();
      if(log_file_open_flag == 1)
      {
         data = new TFileOpenDialog::TData(
            OFN_HIDEREADONLY, "LOG files (*.log)|*.log|",
            0,
            0,
            "LOG"
         );
         if ((new TFileOpenDialog(this, *data))->Execute() == IDOK)
         {
            if(cp && ID_MYTIMER)
            {
               timecount = 0;
               cp->SaveFile();
            }
            strcpy(filen, data->FileName);
            if(cp != NULL)
            {
               cp->SetCaption(filen);
               TModule *TM;
               TM = GetModule();
               cp->filename = filen;
               cp->sArray.Flush();
               cp->numRows = currentRows = 0;
               ID_MYTIMER = ::SetTimer(NULL, 1, 31000, (TIMERPROC)MakeProcInstance((FARPROC)Save, TM->GetInstance()));
            }      
         }
      }
      else if(!log_file_open_flag)
      {
         strcpy(filen, ""); //no log file saving
         if(cp)
         {
            cp->SetCaption("");
            if(ID_MYTIMER)
            {
               timecount = 0;
               cp->SaveFile();  //save the log file if it needs it
               ::KillTimer(NULL, ID_MYTIMER); // kill the save timer
            }
         }
      }
   }
   else
      BWCCMessageBox(NULL, "DS1993 not present or system busy, Function denied", "Error", MB_OK);
}
void TMicroLanClient::CmDiagnostics()
{
   if(!TMValidSession(handle))
   {
      TMEndSession(handle);
      handle = TMStartSession(comport);
      TMSetup(handle);
   }
   int i = TMSetup(handle);
   if(i == 1)
      BWCCMessageBox(NULL, "System OK", "Diagnostics", MB_ICONINFORMATION);
   else if(i == 2)
      BWCCMessageBox(NULL, "The MicroLan is shorted or the Com Port Adapter is not connected.", "Diagnostics", MB_ICONHAND);
}

// Display help message box
void TMicroLanClient::CmHelpAbout()
{
   (new TAboutDialog(this, DIALOG_1))->Execute();
}

void TMicroLanClient::CmLogOn()
{
   if(!already_loging)
   {
      CmFileOpen();
      already_loging = TRUE;
      kill_timer_flag = FALSE;
   }
   //   SetTimer(NULL, global_time_interval*1000);
}
void TMicroLanClient::CmLogOff()
{
   if(check_1993() || !alarm_enable)
   {
      if(cp)
      {
         if(ID_MYTIMER != NULL)
         {
            log_file_open_flag = 0;
            strcpy(filen, "");
            timecount = 0;
            cp->SaveFile();
            ::KillTimer(NULL, ID_MYTIMER);
            ID_MYTIMER = FALSE;
         }
         cp->SetCaption("");
      }
   }
   else
      BWCCMessageBox(NULL, "DS1993 not present or system busy, Function denied", "Error", MB_OK);
}

void TMicroLanClient::CmAlarmOn()
{
   int i, j;
   if((i=check_1993()) && !(j = AlarmGadget->GetButtonState()))
   {
      AlarmGadget->SetButtonState(2);
      alarm_enable = FALSE;
      SwitchOff(alarm_cntl);
   }
   else if(i && j == 2)
   {
      AlarmGadget->SetButtonState(0);
      alarm_enable = TRUE;
      write_1993();

   }
   else if(!i)
      BWCCMessageBox(NULL, "DS1993 not present or system busy, Function denied", "Error", MB_OK);
}


void TMicroLanClient::CmExit()
{
   if(check_1993() || !alarm_enable)
   {
      out_stream = open("exp5.stp", O_CREAT|O_TRUNC|O_RDWR|O_BINARY, S_IREAD|S_IWRITE);
      if(out_stream > 0)
         save_setup(root);
      close(out_stream);
   
      if(timer_set_flag)
      cp->KillTimer(1);
      if(ID_MYTIMER != NULL)
      {
         ::KillTimer(NULL, ID_MYTIMER);
         timecount = 0;
         cp->SaveFile();
      }
      if(root)
         deltree(root);
      root = 0;
      PostQuitMessage(0);
   }
   else
      BWCCMessageBox(NULL, "DS1993 not present or system busy, Function denied", "Error", MB_OK);
}

void TMicroLanClient::CmComSetup()
{
   char testbuff[10];
   if(check_1993() || !alarm_enable)
   {
      TComSetupDlg TCom(this, DIALOG_4);
      if(TCom.Execute() == IDOK)
      {
         itoa(comport, testbuff, 10);
         testbuff[1] = '0';
         BWCCMessageBox(NULL, testbuff, "ComPort", MB_OK);
      }
   }
   else
      BWCCMessageBox(NULL, "DS1993 not present or system busy, Function denied", "Error", MB_OK);
}

void TMicroLanClient::CmChange_Temp()
{
   if(check_1993() || !alarm_enable)
   {
      TChangeTempDlg TChange(this, CHANGE_TEMP);
      TChange.Execute();
   }
   else 
      BWCCMessageBox(NULL, "DS1993 not present or system busy, Function denied", "Error", MB_OK);
}
// ===========================================================
// The MDI application class
// ===========================================================

class TMicroLanApp: public TApplication {
public:
  TMicroLanApp(const char far* name)
    : TApplication(name) {};
  void InitMainWindow();
  bool CanClose();
private:
  TMicroLanClient* client;
  TControlBar* toolbar;
  TStatusBar* statusLine;
};

// Initialize the program's main window
void TMicroLanApp::InitMainWindow()
{
  
// Construct the MDI client window
  client = new TMicroLanClient;

// Construct MDI frame, passing client window object
////////////////////////////////////////////////////////////////

  TDecoratedMDIFrame * frame = new TDecoratedMDIFrame(
    "Burglar Alarm and Temperature Control",  // Caption
    MENU_1,      // Required menu resource identifier
    *client,      // Required client window object
    TRUE);        // Optional track menu selections


// Construct a toolbar with icons for various commands
   toolbar = new TControlBar(frame);
   toolbar->Insert(
      *new TButtonGadget(LOGON, CM_LOGON));
   toolbar->Insert(
      *new TButtonGadget(LOGOFF, CM_LOGOFF));
   toolbar->Insert(
      *new TSeparatorGadget(16));
   toolbar->Insert(
      *new TButtonGadget(CM_SETUP, CM_LOGSETUP));
   toolbar->Insert(
      *new TButtonGadget(CHANGE_TEMP, CM_CHANGETEMP));
   toolbar->Insert(
      *new TButtonGadget(IDB_DIAGNOSTICS, CM_DIAGNOSTICS));
   toolbar->Insert(
      *new TSeparatorGadget(16));
   toolbar->Insert(*(AlarmGadget = new TButtonGadget(ALARM_ENABLE, CM_ALARMON)));
   toolbar->Insert(
      *new TSeparatorGadget(16));
   toolbar->Insert(
      *new TButtonGadget(IDB_INFO, CM_HELPABOUT));
   toolbar->Insert(
      *new TButtonGadget(IDB_FILEEXIT, CM_EXIT));
   toolbar->Insert(
      *new TSeparatorGadget(8));

// Enable status line messages when mouse touches a button
  toolbar->SetHintMode(TGadgetWindow::EnterHints);

   
// Insert the toolbar into the frame window
  frame->Insert(*toolbar, TDecoratedFrame::Top);

// Create status line with function keys legend
  statusLine = new TStatusBar(frame, TGadget::Raised);
  frame->Insert(*statusLine, TDecoratedFrame::Bottom);
// Assign the decorated MDI frame as the program's main window
  SetMainWindow(frame);
  MainWindow->SetIcon(this, A_N_T_ICON);
  strcpy(filen, "");
}

bool TMicroLanApp::CanClose()
{
   if(check_1993() || !alarm_enable)
   {
      TApplication::CanClose();
      if(timer_set_flag)
         cp->KillTimer(1);
      if(ID_MYTIMER != NULL)
      {
         ::KillTimer(NULL, ID_MYTIMER);
         timecount = 0;
         cp->SaveFile();
      }
      if(root)
         deltree(root);
      root = 0;
      return TRUE;
   }
   else
   {
      BWCCMessageBox(NULL, "DS1993 not present or system busy, Function denied", "Error", MB_OK);
      return FALSE;
   }
}
#pragma argsused

// Main program
int OwlMain(int argc, char* argv[])
{
   if(argc > 1)
   {
      comport = atoi(argv[1]);
      if(comport < 1 || comport > 16)
         comport = 0;    
   }
   else
      comport = 0;
   TMicroLanApp app("Burglar Alarm and Temperature Control");
   return app.Run();
}

